# Generated from romanian.sbl by Snowball 3.1.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class RomanianStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from romanian.sbl by Snowball 3.1.1 - https://snowballstem.org/
    '''

    g_v = {"a", "e", "i", "o", "u", "â", "î", "ă"}

    B_standard_suffix_removed = False
    I_p2 = 0
    I_p1 = 0
    I_pV = 0

    def __r_norm(self):
        v_1 = self.cursor
        try:
            while True:
                v_2 = self.cursor
                try:
                    while True:
                        v_3 = self.cursor
                        try:
                            self.bra = self.cursor
                            among_var = self.find_among(RomanianStemmer.a_0)
                            if among_var == 0:
                                raise lab2()
                            self.ket = self.cursor
                            self.slice_from(RomanianStemmer.as_0[among_var - 1])
                            self.cursor = v_3
                            break
                        except lab2: pass
                        self.cursor = v_3
                        if self.cursor >= self.limit:
                            raise lab1()
                        self.cursor += 1
                    continue
                except lab1: pass
                self.cursor = v_2
                break
        except lab0: pass
        self.cursor = v_1
        return True

    def __r_prelude(self):
        while True:
            v_1 = self.cursor
            try:
                while True:
                    v_2 = self.cursor
                    try:
                        if not self.in_grouping(RomanianStemmer.g_v):
                            raise lab1()
                        self.bra = self.cursor
                        while True:
                            v_3 = self.cursor
                            try:
                                if self.cursor == self.limit or self.current[self.cursor] != "u":
                                    raise lab2()
                                self.cursor += 1
                                self.ket = self.cursor
                                if not self.in_grouping(RomanianStemmer.g_v):
                                    raise lab2()
                                self.slice_from("U")
                                break
                            except lab2: pass
                            self.cursor = v_3
                            if self.cursor == self.limit or self.current[self.cursor] != "i":
                                raise lab1()
                            self.cursor += 1
                            self.ket = self.cursor
                            if not self.in_grouping(RomanianStemmer.g_v):
                                raise lab1()
                            self.slice_from("I")
                            break
                        self.cursor = v_2
                        break
                    except lab1: pass
                    self.cursor = v_2
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_mark_regions(self):
        self.I_pV = self.limit
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            while True:
                v_2 = self.cursor
                try:
                    if not self.in_grouping(RomanianStemmer.g_v):
                        raise lab1()
                    while True:
                        v_3 = self.cursor
                        try:
                            if not self.out_grouping(RomanianStemmer.g_v):
                                raise lab2()
                            if not self.go_out_grouping(RomanianStemmer.g_v):
                                raise lab2()
                            self.cursor += 1
                            break
                        except lab2: pass
                        self.cursor = v_3
                        if not self.in_grouping(RomanianStemmer.g_v):
                            raise lab1()
                        if not self.go_in_grouping(RomanianStemmer.g_v):
                            raise lab1()
                        self.cursor += 1
                        break
                    break
                except lab1: pass
                self.cursor = v_2
                if not self.out_grouping(RomanianStemmer.g_v):
                    raise lab0()
                while True:
                    v_4 = self.cursor
                    try:
                        if not self.out_grouping(RomanianStemmer.g_v):
                            raise lab1()
                        if not self.go_out_grouping(RomanianStemmer.g_v):
                            raise lab1()
                        self.cursor += 1
                        break
                    except lab1: pass
                    self.cursor = v_4
                    if not self.in_grouping(RomanianStemmer.g_v):
                        raise lab0()
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                    break
                break
            self.I_pV = self.cursor
        except lab0: pass
        self.cursor = v_1
        v_5 = self.cursor
        try:
            if not self.go_out_grouping(RomanianStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(RomanianStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p1 = self.cursor
            if not self.go_out_grouping(RomanianStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(RomanianStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p2 = self.cursor
        except lab0: pass
        self.cursor = v_5
        return True

    def __r_postlude(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(RomanianStemmer.a_1)
                self.ket = self.cursor
                if among_var == 1:
                    self.slice_from("i")
                elif among_var == 2:
                    self.slice_from("u")
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_R1(self):
        return self.I_p1 <= self.cursor

    def __r_step_0(self):
        self.ket = self.cursor
        among_var = self.find_among_b(RomanianStemmer.a_2)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if among_var == 1:
            self.slice_del()
        elif among_var == 2:
            self.slice_from("a")
        elif among_var == 3:
            self.slice_from("e")
        elif among_var == 4:
            self.slice_from("i")
        elif among_var == 5:
            try:
                if not self.eq_s_b("ab"):
                    raise lab0()
                return False
            except lab0: pass
            self.slice_from("i")
        elif among_var == 6:
            self.slice_from("at")
        else:
            self.slice_from("ați")
        return True

    def __r_combo_suffix(self):
        v_1 = self.limit - self.cursor
        self.ket = self.cursor
        among_var = self.find_among_b(RomanianStemmer.a_3)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        self.slice_from(RomanianStemmer.as_3[among_var - 1])
        self.B_standard_suffix_removed = True
        self.cursor = self.limit - v_1
        return True

    def __r_standard_suffix(self):
        self.B_standard_suffix_removed = False
        while True:
            v_1 = self.limit - self.cursor
            try:
                if not self.__r_combo_suffix():
                    raise lab0()
                continue
            except lab0: pass
            self.cursor = self.limit - v_1
            break
        self.ket = self.cursor
        among_var = self.find_among_b(RomanianStemmer.a_4)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if self.I_p2 > self.cursor:
            return False
        if among_var == 1:
            self.slice_del()
        elif among_var == 2:
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "ț":
                return False
            self.cursor -= 1
            self.bra = self.cursor
            self.slice_from("t")
        else:
            self.slice_from("ist")
        self.B_standard_suffix_removed = True
        return True

    def __r_verb_suffix(self):
        if self.cursor < self.I_pV:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_pV
        self.ket = self.cursor
        among_var = self.find_among_b(RomanianStemmer.a_5)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        if among_var == 1:
            while True:
                try:
                    if not self.out_grouping_b(RomanianStemmer.g_v):
                        raise lab0()
                    break
                except lab0: pass
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "u":
                    self.limit_backward = v_2
                    return False
                self.cursor -= 1
                break
            self.slice_del()
        else:
            self.slice_del()
        self.limit_backward = v_2
        return True

    def __r_vowel_suffix(self):
        self.ket = self.cursor
        if self.find_among_b(RomanianStemmer.a_6) == 0:
            return False
        self.bra = self.cursor
        if self.I_pV > self.cursor:
            return False
        self.slice_del()
        return True

    def _stem(self):
        self.__r_norm()
        v_1 = self.cursor
        self.__r_prelude()
        self.cursor = v_1
        self.__r_mark_regions()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_2 = self.limit - self.cursor
        self.__r_step_0()
        self.cursor = self.limit - v_2
        v_3 = self.limit - self.cursor
        self.__r_standard_suffix()
        self.cursor = self.limit - v_3
        v_4 = self.limit - self.cursor
        try:
            while True:
                try:
                    if not self.B_standard_suffix_removed:
                        raise lab1()
                    break
                except lab1: pass
                if not self.__r_verb_suffix():
                    raise lab0()
                break
        except lab0: pass
        self.cursor = self.limit - v_4
        v_5 = self.limit - self.cursor
        self.__r_vowel_suffix()
        self.cursor = self.limit - v_5
        self.cursor = self.limit_backward
        v_6 = self.cursor
        self.__r_postlude()
        self.cursor = v_6
        return True

    a_0 = [
        Among("ş", -1, 1),
        Among("ţ", -1, 2)
    ]
    as_0 = ("ș", "ț")

    a_1 = [
        Among("", -1, 3),
        Among("I", 0, 1),
        Among("U", 0, 2)
    ]

    a_2 = [
        Among("ea", -1, 3),
        Among("ația", -1, 7),
        Among("aua", -1, 2),
        Among("iua", -1, 4),
        Among("ație", -1, 7),
        Among("ele", -1, 3),
        Among("ile", -1, 5),
        Among("iile", 6, 4),
        Among("iei", -1, 4),
        Among("atei", -1, 6),
        Among("ii", -1, 4),
        Among("ului", -1, 1),
        Among("ul", -1, 1),
        Among("elor", -1, 3),
        Among("ilor", -1, 4),
        Among("iilor", 14, 4)
    ]

    a_3 = [
        Among("icala", -1, 4),
        Among("iciva", -1, 4),
        Among("ativa", -1, 5),
        Among("itiva", -1, 6),
        Among("icale", -1, 4),
        Among("ațiune", -1, 5),
        Among("ițiune", -1, 6),
        Among("atoare", -1, 5),
        Among("itoare", -1, 6),
        Among("ătoare", -1, 5),
        Among("icitate", -1, 4),
        Among("abilitate", -1, 1),
        Among("ibilitate", -1, 2),
        Among("ivitate", -1, 3),
        Among("icive", -1, 4),
        Among("ative", -1, 5),
        Among("itive", -1, 6),
        Among("icali", -1, 4),
        Among("atori", -1, 5),
        Among("icatori", 18, 4),
        Among("itori", -1, 6),
        Among("ători", -1, 5),
        Among("icitati", -1, 4),
        Among("abilitati", -1, 1),
        Among("ivitati", -1, 3),
        Among("icivi", -1, 4),
        Among("ativi", -1, 5),
        Among("itivi", -1, 6),
        Among("icităi", -1, 4),
        Among("abilităi", -1, 1),
        Among("ivităi", -1, 3),
        Among("icități", -1, 4),
        Among("abilități", -1, 1),
        Among("ivități", -1, 3),
        Among("ical", -1, 4),
        Among("ator", -1, 5),
        Among("icator", 35, 4),
        Among("itor", -1, 6),
        Among("ător", -1, 5),
        Among("iciv", -1, 4),
        Among("ativ", -1, 5),
        Among("itiv", -1, 6),
        Among("icală", -1, 4),
        Among("icivă", -1, 4),
        Among("ativă", -1, 5),
        Among("itivă", -1, 6)
    ]
    as_3 = ("abil", "ibil", "iv", "ic", "at", "it")

    a_4 = [
        Among("ica", -1, 1),
        Among("abila", -1, 1),
        Among("ibila", -1, 1),
        Among("oasa", -1, 1),
        Among("ata", -1, 1),
        Among("ita", -1, 1),
        Among("anta", -1, 1),
        Among("ista", -1, 3),
        Among("uta", -1, 1),
        Among("iva", -1, 1),
        Among("ic", -1, 1),
        Among("ice", -1, 1),
        Among("abile", -1, 1),
        Among("ibile", -1, 1),
        Among("isme", -1, 3),
        Among("iune", -1, 2),
        Among("oase", -1, 1),
        Among("ate", -1, 1),
        Among("itate", 17, 1),
        Among("ite", -1, 1),
        Among("ante", -1, 1),
        Among("iste", -1, 3),
        Among("ute", -1, 1),
        Among("ive", -1, 1),
        Among("ici", -1, 1),
        Among("abili", -1, 1),
        Among("ibili", -1, 1),
        Among("iuni", -1, 2),
        Among("atori", -1, 1),
        Among("osi", -1, 1),
        Among("ati", -1, 1),
        Among("itati", 30, 1),
        Among("iti", -1, 1),
        Among("anti", -1, 1),
        Among("isti", -1, 3),
        Among("uti", -1, 1),
        Among("iști", -1, 3),
        Among("ivi", -1, 1),
        Among("ităi", -1, 1),
        Among("oși", -1, 1),
        Among("ități", -1, 1),
        Among("abil", -1, 1),
        Among("ibil", -1, 1),
        Among("ism", -1, 3),
        Among("ator", -1, 1),
        Among("os", -1, 1),
        Among("at", -1, 1),
        Among("it", -1, 1),
        Among("ant", -1, 1),
        Among("ist", -1, 3),
        Among("ut", -1, 1),
        Among("iv", -1, 1),
        Among("ică", -1, 1),
        Among("abilă", -1, 1),
        Among("ibilă", -1, 1),
        Among("oasă", -1, 1),
        Among("ată", -1, 1),
        Among("ită", -1, 1),
        Among("antă", -1, 1),
        Among("istă", -1, 3),
        Among("ută", -1, 1),
        Among("ivă", -1, 1)
    ]

    a_5 = [
        Among("ea", -1, 1),
        Among("ia", -1, 1),
        Among("esc", -1, 1),
        Among("ăsc", -1, 1),
        Among("ind", -1, 1),
        Among("ând", -1, 1),
        Among("are", -1, 1),
        Among("ere", -1, 1),
        Among("ire", -1, 1),
        Among("âre", -1, 1),
        Among("se", -1, 2),
        Among("ase", 10, 1),
        Among("sese", 10, 2),
        Among("ise", 10, 1),
        Among("use", 10, 1),
        Among("âse", 10, 1),
        Among("ește", -1, 1),
        Among("ăște", -1, 1),
        Among("eze", -1, 1),
        Among("ai", -1, 1),
        Among("eai", 19, 1),
        Among("iai", 19, 1),
        Among("sei", -1, 2),
        Among("ești", -1, 1),
        Among("ăști", -1, 1),
        Among("ui", -1, 1),
        Among("ezi", -1, 1),
        Among("âi", -1, 1),
        Among("ași", -1, 1),
        Among("seși", -1, 2),
        Among("aseși", 29, 1),
        Among("seseși", 29, 2),
        Among("iseși", 29, 1),
        Among("useși", 29, 1),
        Among("âseși", 29, 1),
        Among("iși", -1, 1),
        Among("uși", -1, 1),
        Among("âși", -1, 1),
        Among("ați", -1, 2),
        Among("eați", 38, 1),
        Among("iați", 38, 1),
        Among("eți", -1, 2),
        Among("iți", -1, 2),
        Among("âți", -1, 2),
        Among("arăți", -1, 1),
        Among("serăți", -1, 2),
        Among("aserăți", 45, 1),
        Among("seserăți", 45, 2),
        Among("iserăți", 45, 1),
        Among("userăți", 45, 1),
        Among("âserăți", 45, 1),
        Among("irăți", -1, 1),
        Among("urăți", -1, 1),
        Among("ârăți", -1, 1),
        Among("am", -1, 1),
        Among("eam", 54, 1),
        Among("iam", 54, 1),
        Among("em", -1, 2),
        Among("asem", 57, 1),
        Among("sesem", 57, 2),
        Among("isem", 57, 1),
        Among("usem", 57, 1),
        Among("âsem", 57, 1),
        Among("im", -1, 2),
        Among("âm", -1, 2),
        Among("ăm", -1, 2),
        Among("arăm", 65, 1),
        Among("serăm", 65, 2),
        Among("aserăm", 67, 1),
        Among("seserăm", 67, 2),
        Among("iserăm", 67, 1),
        Among("userăm", 67, 1),
        Among("âserăm", 67, 1),
        Among("irăm", 65, 1),
        Among("urăm", 65, 1),
        Among("ârăm", 65, 1),
        Among("au", -1, 1),
        Among("eau", 76, 1),
        Among("iau", 76, 1),
        Among("indu", -1, 1),
        Among("ându", -1, 1),
        Among("ez", -1, 1),
        Among("ească", -1, 1),
        Among("ară", -1, 1),
        Among("seră", -1, 2),
        Among("aseră", 84, 1),
        Among("seseră", 84, 2),
        Among("iseră", 84, 1),
        Among("useră", 84, 1),
        Among("âseră", 84, 1),
        Among("iră", -1, 1),
        Among("ură", -1, 1),
        Among("âră", -1, 1),
        Among("ează", -1, 1)
    ]

    a_6 = [
        Among("a", -1, 1),
        Among("e", -1, 1),
        Among("ie", 1, 1),
        Among("i", -1, 1),
        Among("ă", -1, 1)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass



# Generated from arabic.sbl by Snowball 3.1.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class ArabicStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from arabic.sbl by Snowball 3.1.1 - https://snowballstem.org/
    '''

    B_is_defined = False
    B_is_verb = False
    B_is_noun = False

    def __r_Normalize_pre(self):
        v_1 = self.cursor
        try:
            while True:
                v_2 = self.cursor
                try:
                    while True:
                        v_3 = self.cursor
                        try:
                            self.bra = self.cursor
                            among_var = self.find_among(ArabicStemmer.a_0)
                            if among_var == 0:
                                raise lab2()
                            self.ket = self.cursor
                            self.slice_from(ArabicStemmer.as_0[among_var - 1])
                            break
                        except lab2: pass
                        self.cursor = v_3
                        if self.cursor >= self.limit:
                            raise lab1()
                        self.cursor += 1
                        break
                    continue
                except lab1: pass
                self.cursor = v_2
                break
        except lab0: pass
        self.cursor = v_1
        return True

    def __r_Normalize_post(self):
        v_1 = self.cursor
        try:
            self.limit_backward = self.cursor
            self.cursor = self.limit
            self.ket = self.cursor
            if self.find_among_b(ArabicStemmer.a_1) == 0:
                raise lab0()
            self.bra = self.cursor
            self.slice_from("\u0621")
            self.cursor = self.limit_backward
        except lab0: pass
        self.cursor = v_1
        v_2 = self.cursor
        try:
            while True:
                v_3 = self.cursor
                try:
                    while True:
                        v_4 = self.cursor
                        try:
                            self.bra = self.cursor
                            among_var = self.find_among(ArabicStemmer.a_2)
                            if among_var == 0:
                                raise lab2()
                            self.ket = self.cursor
                            self.slice_from(ArabicStemmer.as_2[among_var - 1])
                            break
                        except lab2: pass
                        self.cursor = v_4
                        if self.cursor >= self.limit:
                            raise lab1()
                        self.cursor += 1
                        break
                    continue
                except lab1: pass
                self.cursor = v_3
                break
        except lab0: pass
        self.cursor = v_2
        return True

    def __r_Checks1(self):
        self.bra = self.cursor
        among_var = self.find_among(ArabicStemmer.a_3)
        if among_var == 0:
            return False
        self.ket = self.cursor
        if among_var == 1:
            if len(self.current) <= 4:
                return False
            self.B_is_noun = True
            self.B_is_verb = False
            self.B_is_defined = True
        else:
            if len(self.current) <= 3:
                return False
            self.B_is_noun = True
            self.B_is_verb = False
            self.B_is_defined = True
        return True

    def __r_Prefix_Step1(self):
        self.bra = self.cursor
        among_var = self.find_among(ArabicStemmer.a_4)
        if among_var == 0:
            return False
        self.ket = self.cursor
        if among_var == 1:
            if len(self.current) <= 3:
                return False
            self.slice_from("\u0623")
        elif among_var == 2:
            if len(self.current) <= 3:
                return False
            self.slice_from("\u0622")
        elif among_var == 3:
            if len(self.current) <= 3:
                return False
            self.slice_from("\u0627")
        else:
            if len(self.current) <= 3:
                return False
            self.slice_from("\u0625")
        return True

    def __r_Prefix_Step2(self):
        self.bra = self.cursor
        if self.find_among(ArabicStemmer.a_5) == 0:
            return False
        self.ket = self.cursor
        if len(self.current) <= 3:
            return False
        try:
            if self.cursor == self.limit or self.current[self.cursor] != "\u0627":
                raise lab0()
            self.cursor += 1
            return False
        except lab0: pass
        self.slice_del()
        return True

    def __r_Prefix_Step3a_Noun(self):
        self.bra = self.cursor
        among_var = self.find_among(ArabicStemmer.a_6)
        if among_var == 0:
            return False
        self.ket = self.cursor
        if among_var == 1:
            if len(self.current) <= 5:
                return False
            self.slice_del()
        else:
            if len(self.current) <= 4:
                return False
            self.slice_del()
        return True

    def __r_Prefix_Step3b_Noun(self):
        self.bra = self.cursor
        among_var = self.find_among(ArabicStemmer.a_7)
        if among_var == 0:
            return False
        self.ket = self.cursor
        if among_var == 1:
            if len(self.current) <= 3:
                return False
            self.slice_del()
        elif among_var == 2:
            if len(self.current) <= 3:
                return False
            self.slice_from("\u0628")
        elif among_var == 3:
            if len(self.current) <= 3:
                return False
            self.slice_from("\u0643")
        return True

    def __r_Prefix_Step3_Verb(self):
        self.bra = self.cursor
        among_var = self.find_among(ArabicStemmer.a_8)
        if among_var == 0:
            return False
        self.ket = self.cursor
        if among_var == 1:
            if len(self.current) <= 4:
                return False
            self.slice_from("\u064A")
        elif among_var == 2:
            if len(self.current) <= 4:
                return False
            self.slice_from("\u062A")
        elif among_var == 3:
            if len(self.current) <= 4:
                return False
            self.slice_from("\u0646")
        else:
            if len(self.current) <= 4:
                return False
            self.slice_from("\u0623")
        return True

    def __r_Prefix_Step4_Verb(self):
        self.bra = self.cursor
        if self.find_among(ArabicStemmer.a_9) == 0:
            return False
        self.ket = self.cursor
        if len(self.current) <= 4:
            return False
        self.B_is_verb = True
        self.B_is_noun = False
        self.slice_from("\u0627\u0633\u062A")
        return True

    def __r_Suffix_Noun_Step1a(self):
        self.ket = self.cursor
        among_var = self.find_among_b(ArabicStemmer.a_10)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if len(self.current) < 4:
                return False
            self.slice_del()
        elif among_var == 2:
            if len(self.current) < 5:
                return False
            self.slice_del()
        else:
            if len(self.current) < 6:
                return False
            self.slice_del()
        return True

    def __r_Suffix_Noun_Step1b(self):
        self.ket = self.cursor
        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "\u0646":
            return False
        self.cursor -= 1
        self.bra = self.cursor
        if len(self.current) <= 5:
            return False
        self.slice_del()
        return True

    def __r_Suffix_Noun_Step2a(self):
        self.ket = self.cursor
        if self.find_among_b(ArabicStemmer.a_11) == 0:
            return False
        self.bra = self.cursor
        if len(self.current) <= 4:
            return False
        self.slice_del()
        return True

    def __r_Suffix_Noun_Step2b(self):
        self.ket = self.cursor
        if not self.eq_s_b("\u0627\u062A"):
            return False
        self.bra = self.cursor
        if len(self.current) < 5:
            return False
        self.slice_del()
        return True

    def __r_Suffix_Noun_Step2c1(self):
        self.ket = self.cursor
        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "\u062A":
            return False
        self.cursor -= 1
        self.bra = self.cursor
        if len(self.current) < 4:
            return False
        self.slice_del()
        return True

    def __r_Suffix_Noun_Step2c2(self):
        self.ket = self.cursor
        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "\u0629":
            return False
        self.cursor -= 1
        self.bra = self.cursor
        if len(self.current) < 4:
            return False
        self.slice_del()
        return True

    def __r_Suffix_Noun_Step3(self):
        self.ket = self.cursor
        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "\u064A":
            return False
        self.cursor -= 1
        self.bra = self.cursor
        if len(self.current) < 3:
            return False
        self.slice_del()
        return True

    def __r_Suffix_Verb_Step1(self):
        self.ket = self.cursor
        among_var = self.find_among_b(ArabicStemmer.a_12)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if len(self.current) < 4:
                return False
            self.slice_del()
        elif among_var == 2:
            if len(self.current) < 5:
                return False
            self.slice_del()
        else:
            if len(self.current) < 6:
                return False
            self.slice_del()
        return True

    def __r_Suffix_Verb_Step2a(self):
        self.ket = self.cursor
        among_var = self.find_among_b(ArabicStemmer.a_13)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if len(self.current) < 4:
                return False
            self.slice_del()
        elif among_var == 2:
            if len(self.current) < 5:
                return False
            self.slice_del()
        elif among_var == 3:
            if len(self.current) <= 5:
                return False
            self.slice_del()
        else:
            if len(self.current) < 6:
                return False
            self.slice_del()
        return True

    def __r_Suffix_Verb_Step2b(self):
        self.ket = self.cursor
        if self.find_among_b(ArabicStemmer.a_14) == 0:
            return False
        self.bra = self.cursor
        if len(self.current) < 5:
            return False
        self.slice_del()
        return True

    def __r_Suffix_Verb_Step2c(self):
        self.ket = self.cursor
        among_var = self.find_among_b(ArabicStemmer.a_15)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if len(self.current) < 4:
                return False
            self.slice_del()
        else:
            if len(self.current) < 6:
                return False
            self.slice_del()
        return True

    def __r_Suffix_All_alef_maqsura(self):
        self.ket = self.cursor
        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "\u0649":
            return False
        self.cursor -= 1
        self.bra = self.cursor
        self.slice_from("\u064A")
        return True

    def _stem(self):
        self.B_is_noun = True
        self.B_is_verb = True
        self.B_is_defined = False
        v_1 = self.cursor
        self.__r_Checks1()
        self.cursor = v_1
        self.__r_Normalize_pre()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_2 = self.limit - self.cursor
        try:
            while True:
                v_3 = self.limit - self.cursor
                try:
                    if not self.B_is_verb:
                        raise lab1()
                    while True:
                        v_4 = self.limit - self.cursor
                        try:
                            v_5 = 1
                            while True:
                                v_6 = self.limit - self.cursor
                                try:
                                    if not self.__r_Suffix_Verb_Step1():
                                        raise lab3()
                                    v_5 -= 1
                                    continue
                                except lab3: pass
                                self.cursor = self.limit - v_6
                                break
                            if v_5 > 0:
                                raise lab2()
                            while True:
                                v_7 = self.limit - self.cursor
                                try:
                                    if not self.__r_Suffix_Verb_Step2a():
                                        raise lab3()
                                    break
                                except lab3: pass
                                self.cursor = self.limit - v_7
                                try:
                                    if not self.__r_Suffix_Verb_Step2c():
                                        raise lab3()
                                    break
                                except lab3: pass
                                self.cursor = self.limit - v_7
                                if self.cursor <= self.limit_backward:
                                    raise lab2()
                                self.cursor -= 1
                                break
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_4
                        try:
                            if not self.__r_Suffix_Verb_Step2b():
                                raise lab2()
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_4
                        if not self.__r_Suffix_Verb_Step2a():
                            raise lab1()
                        break
                    break
                except lab1: pass
                self.cursor = self.limit - v_3
                try:
                    if not self.B_is_noun:
                        raise lab1()
                    v_8 = self.limit - self.cursor
                    try:
                        while True:
                            v_9 = self.limit - self.cursor
                            try:
                                if not self.__r_Suffix_Noun_Step2c2():
                                    raise lab3()
                                break
                            except lab3: pass
                            self.cursor = self.limit - v_9
                            try:
                                if self.B_is_defined:
                                    raise lab3()
                                if not self.__r_Suffix_Noun_Step1a():
                                    raise lab3()
                                while True:
                                    v_10 = self.limit - self.cursor
                                    try:
                                        if not self.__r_Suffix_Noun_Step2a():
                                            raise lab4()
                                        break
                                    except lab4: pass
                                    self.cursor = self.limit - v_10
                                    try:
                                        if not self.__r_Suffix_Noun_Step2b():
                                            raise lab4()
                                        break
                                    except lab4: pass
                                    self.cursor = self.limit - v_10
                                    try:
                                        if not self.__r_Suffix_Noun_Step2c1():
                                            raise lab4()
                                        break
                                    except lab4: pass
                                    self.cursor = self.limit - v_10
                                    if self.cursor <= self.limit_backward:
                                        raise lab3()
                                    self.cursor -= 1
                                    break
                                break
                            except lab3: pass
                            self.cursor = self.limit - v_9
                            try:
                                if not self.__r_Suffix_Noun_Step1b():
                                    raise lab3()
                                while True:
                                    v_11 = self.limit - self.cursor
                                    try:
                                        if not self.__r_Suffix_Noun_Step2a():
                                            raise lab4()
                                        break
                                    except lab4: pass
                                    self.cursor = self.limit - v_11
                                    try:
                                        if not self.__r_Suffix_Noun_Step2b():
                                            raise lab4()
                                        break
                                    except lab4: pass
                                    self.cursor = self.limit - v_11
                                    if not self.__r_Suffix_Noun_Step2c1():
                                        raise lab3()
                                    break
                                break
                            except lab3: pass
                            self.cursor = self.limit - v_9
                            try:
                                if self.B_is_defined:
                                    raise lab3()
                                if not self.__r_Suffix_Noun_Step2a():
                                    raise lab3()
                                break
                            except lab3: pass
                            self.cursor = self.limit - v_9
                            if not self.__r_Suffix_Noun_Step2b():
                                self.cursor = self.limit - v_8
                                raise lab2()
                            break
                    except lab2: pass
                    if not self.__r_Suffix_Noun_Step3():
                        raise lab1()
                    break
                except lab1: pass
                self.cursor = self.limit - v_3
                if not self.__r_Suffix_All_alef_maqsura():
                    raise lab0()
                break
        except lab0: pass
        self.cursor = self.limit - v_2
        self.cursor = self.limit_backward
        v_12 = self.cursor
        try:
            v_13 = self.cursor
            try:
                if not self.__r_Prefix_Step1():
                    self.cursor = v_13
                    raise lab1()
            except lab1: pass
            v_14 = self.cursor
            try:
                if not self.__r_Prefix_Step2():
                    self.cursor = v_14
                    raise lab1()
            except lab1: pass
            while True:
                v_15 = self.cursor
                try:
                    if not self.__r_Prefix_Step3a_Noun():
                        raise lab1()
                    break
                except lab1: pass
                self.cursor = v_15
                try:
                    if not self.B_is_noun:
                        raise lab1()
                    if not self.__r_Prefix_Step3b_Noun():
                        raise lab1()
                    break
                except lab1: pass
                self.cursor = v_15
                if not self.B_is_verb:
                    raise lab0()
                v_16 = self.cursor
                try:
                    if not self.__r_Prefix_Step3_Verb():
                        self.cursor = v_16
                        raise lab1()
                except lab1: pass
                if not self.__r_Prefix_Step4_Verb():
                    raise lab0()
                break
        except lab0: pass
        self.cursor = v_12
        self.__r_Normalize_post()
        return True

    a_0 = [
        Among("\u0640", -1, 1),
        Among("\u064B", -1, 1),
        Among("\u064C", -1, 1),
        Among("\u064D", -1, 1),
        Among("\u064E", -1, 1),
        Among("\u064F", -1, 1),
        Among("\u0650", -1, 1),
        Among("\u0651", -1, 1),
        Among("\u0652", -1, 1),
        Among("\u0660", -1, 2),
        Among("\u0661", -1, 3),
        Among("\u0662", -1, 4),
        Among("\u0663", -1, 5),
        Among("\u0664", -1, 6),
        Among("\u0665", -1, 7),
        Among("\u0666", -1, 8),
        Among("\u0667", -1, 9),
        Among("\u0668", -1, 10),
        Among("\u0669", -1, 11),
        Among("\uFE80", -1, 12),
        Among("\uFE81", -1, 16),
        Among("\uFE82", -1, 16),
        Among("\uFE83", -1, 13),
        Among("\uFE84", -1, 13),
        Among("\uFE85", -1, 17),
        Among("\uFE86", -1, 17),
        Among("\uFE87", -1, 14),
        Among("\uFE88", -1, 14),
        Among("\uFE89", -1, 15),
        Among("\uFE8A", -1, 15),
        Among("\uFE8B", -1, 15),
        Among("\uFE8C", -1, 15),
        Among("\uFE8D", -1, 18),
        Among("\uFE8E", -1, 18),
        Among("\uFE8F", -1, 19),
        Among("\uFE90", -1, 19),
        Among("\uFE91", -1, 19),
        Among("\uFE92", -1, 19),
        Among("\uFE93", -1, 20),
        Among("\uFE94", -1, 20),
        Among("\uFE95", -1, 21),
        Among("\uFE96", -1, 21),
        Among("\uFE97", -1, 21),
        Among("\uFE98", -1, 21),
        Among("\uFE99", -1, 22),
        Among("\uFE9A", -1, 22),
        Among("\uFE9B", -1, 22),
        Among("\uFE9C", -1, 22),
        Among("\uFE9D", -1, 23),
        Among("\uFE9E", -1, 23),
        Among("\uFE9F", -1, 23),
        Among("\uFEA0", -1, 23),
        Among("\uFEA1", -1, 24),
        Among("\uFEA2", -1, 24),
        Among("\uFEA3", -1, 24),
        Among("\uFEA4", -1, 24),
        Among("\uFEA5", -1, 25),
        Among("\uFEA6", -1, 25),
        Among("\uFEA7", -1, 25),
        Among("\uFEA8", -1, 25),
        Among("\uFEA9", -1, 26),
        Among("\uFEAA", -1, 26),
        Among("\uFEAB", -1, 27),
        Among("\uFEAC", -1, 27),
        Among("\uFEAD", -1, 28),
        Among("\uFEAE", -1, 28),
        Among("\uFEAF", -1, 29),
        Among("\uFEB0", -1, 29),
        Among("\uFEB1", -1, 30),
        Among("\uFEB2", -1, 30),
        Among("\uFEB3", -1, 30),
        Among("\uFEB4", -1, 30),
        Among("\uFEB5", -1, 31),
        Among("\uFEB6", -1, 31),
        Among("\uFEB7", -1, 31),
        Among("\uFEB8", -1, 31),
        Among("\uFEB9", -1, 32),
        Among("\uFEBA", -1, 32),
        Among("\uFEBB", -1, 32),
        Among("\uFEBC", -1, 32),
        Among("\uFEBD", -1, 33),
        Among("\uFEBE", -1, 33),
        Among("\uFEBF", -1, 33),
        Among("\uFEC0", -1, 33),
        Among("\uFEC1", -1, 34),
        Among("\uFEC2", -1, 34),
        Among("\uFEC3", -1, 34),
        Among("\uFEC4", -1, 34),
        Among("\uFEC5", -1, 35),
        Among("\uFEC6", -1, 35),
        Among("\uFEC7", -1, 35),
        Among("\uFEC8", -1, 35),
        Among("\uFEC9", -1, 36),
        Among("\uFECA", -1, 36),
        Among("\uFECB", -1, 36),
        Among("\uFECC", -1, 36),
        Among("\uFECD", -1, 37),
        Among("\uFECE", -1, 37),
        Among("\uFECF", -1, 37),
        Among("\uFED0", -1, 37),
        Among("\uFED1", -1, 38),
        Among("\uFED2", -1, 38),
        Among("\uFED3", -1, 38),
        Among("\uFED4", -1, 38),
        Among("\uFED5", -1, 39),
        Among("\uFED6", -1, 39),
        Among("\uFED7", -1, 39),
        Among("\uFED8", -1, 39),
        Among("\uFED9", -1, 40),
        Among("\uFEDA", -1, 40),
        Among("\uFEDB", -1, 40),
        Among("\uFEDC", -1, 40),
        Among("\uFEDD", -1, 41),
        Among("\uFEDE", -1, 41),
        Among("\uFEDF", -1, 41),
        Among("\uFEE0", -1, 41),
        Among("\uFEE1", -1, 42),
        Among("\uFEE2", -1, 42),
        Among("\uFEE3", -1, 42),
        Among("\uFEE4", -1, 42),
        Among("\uFEE5", -1, 43),
        Among("\uFEE6", -1, 43),
        Among("\uFEE7", -1, 43),
        Among("\uFEE8", -1, 43),
        Among("\uFEE9", -1, 44),
        Among("\uFEEA", -1, 44),
        Among("\uFEEB", -1, 44),
        Among("\uFEEC", -1, 44),
        Among("\uFEED", -1, 45),
        Among("\uFEEE", -1, 45),
        Among("\uFEEF", -1, 46),
        Among("\uFEF0", -1, 46),
        Among("\uFEF1", -1, 47),
        Among("\uFEF2", -1, 47),
        Among("\uFEF3", -1, 47),
        Among("\uFEF4", -1, 47),
        Among("\uFEF5", -1, 51),
        Among("\uFEF6", -1, 51),
        Among("\uFEF7", -1, 49),
        Among("\uFEF8", -1, 49),
        Among("\uFEF9", -1, 50),
        Among("\uFEFA", -1, 50),
        Among("\uFEFB", -1, 48),
        Among("\uFEFC", -1, 48)
    ]
    as_0 = ("", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "\u0621", "\u0623", "\u0625", "\u0626", "\u0622", "\u0624", "\u0627", "\u0628", "\u0629", "\u062A", "\u062B", "\u062C", "\u062D", "\u062E", "\u062F", "\u0630", "\u0631", "\u0632", "\u0633", "\u0634", "\u0635", "\u0636", "\u0637", "\u0638", "\u0639", "\u063A", "\u0641", "\u0642", "\u0643", "\u0644", "\u0645", "\u0646", "\u0647", "\u0648", "\u0649", "\u064A", "\u0644\u0627", "\u0644\u0623", "\u0644\u0625", "\u0644\u0622")

    a_1 = [
        Among("\u0622", -1, 1),
        Among("\u0623", -1, 1),
        Among("\u0624", -1, 1),
        Among("\u0625", -1, 1),
        Among("\u0626", -1, 1)
    ]

    a_2 = [
        Among("\u0622", -1, 1),
        Among("\u0623", -1, 1),
        Among("\u0624", -1, 2),
        Among("\u0625", -1, 1),
        Among("\u0626", -1, 3)
    ]
    as_2 = ("\u0627", "\u0648", "\u064A")

    a_3 = [
        Among("\u0627\u0644", -1, 2),
        Among("\u0628\u0627\u0644", -1, 1),
        Among("\u0643\u0627\u0644", -1, 1),
        Among("\u0644\u0644", -1, 2)
    ]

    a_4 = [
        Among("\u0623\u0622", -1, 2),
        Among("\u0623\u0623", -1, 1),
        Among("\u0623\u0624", -1, 1),
        Among("\u0623\u0625", -1, 4),
        Among("\u0623\u0627", -1, 3)
    ]

    a_5 = [
        Among("\u0641", -1, 1),
        Among("\u0648", -1, 1)
    ]

    a_6 = [
        Among("\u0627\u0644", -1, 2),
        Among("\u0628\u0627\u0644", -1, 1),
        Among("\u0643\u0627\u0644", -1, 1),
        Among("\u0644\u0644", -1, 2)
    ]

    a_7 = [
        Among("\u0628", -1, 1),
        Among("\u0628\u0627", 0, -1),
        Among("\u0628\u0628", 0, 2),
        Among("\u0643\u0643", -1, 3)
    ]

    a_8 = [
        Among("\u0633\u0623", -1, 4),
        Among("\u0633\u062A", -1, 2),
        Among("\u0633\u0646", -1, 3),
        Among("\u0633\u064A", -1, 1)
    ]

    a_9 = [
        Among("\u062A\u0633\u062A", -1, 1),
        Among("\u0646\u0633\u062A", -1, 1),
        Among("\u064A\u0633\u062A", -1, 1)
    ]

    a_10 = [
        Among("\u0643\u0645\u0627", -1, 3),
        Among("\u0647\u0645\u0627", -1, 3),
        Among("\u0646\u0627", -1, 2),
        Among("\u0647\u0627", -1, 2),
        Among("\u0643", -1, 1),
        Among("\u0643\u0645", -1, 2),
        Among("\u0647\u0645", -1, 2),
        Among("\u0647\u0646", -1, 2),
        Among("\u0647", -1, 1),
        Among("\u064A", -1, 1)
    ]

    a_11 = [
        Among("\u0627", -1, 1),
        Among("\u0648", -1, 1),
        Among("\u064A", -1, 1)
    ]

    a_12 = [
        Among("\u0643\u0645\u0627", -1, 3),
        Among("\u0647\u0645\u0627", -1, 3),
        Among("\u0646\u0627", -1, 2),
        Among("\u0647\u0627", -1, 2),
        Among("\u0643", -1, 1),
        Among("\u0643\u0645", -1, 2),
        Among("\u0647\u0645", -1, 2),
        Among("\u0643\u0646", -1, 2),
        Among("\u0647\u0646", -1, 2),
        Among("\u0647", -1, 1),
        Among("\u0643\u0645\u0648", -1, 3),
        Among("\u0646\u064A", -1, 2)
    ]

    a_13 = [
        Among("\u0627", -1, 1),
        Among("\u062A\u0627", 0, 2),
        Among("\u062A\u0645\u0627", 0, 4),
        Among("\u0646\u0627", 0, 2),
        Among("\u062A", -1, 1),
        Among("\u0646", -1, 1),
        Among("\u0627\u0646", 5, 3),
        Among("\u062A\u0646", 5, 2),
        Among("\u0648\u0646", 5, 3),
        Among("\u064A\u0646", 5, 3),
        Among("\u064A", -1, 1)
    ]

    a_14 = [
        Among("\u0648\u0627", -1, 1),
        Among("\u062A\u0645", -1, 1)
    ]

    a_15 = [
        Among("\u0648", -1, 1),
        Among("\u062A\u0645\u0648", 0, 2)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass


class lab4(BaseException): pass



# Generated from esperanto.sbl by Snowball 3.1.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class EsperantoStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from esperanto.sbl by Snowball 3.1.1 - https://snowballstem.org/
    '''

    g_vowel = {"a", "e", "i", "o", "u"}

    g_aou = "aou"
    g_digit = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9"}


    def __r_canonical_form(self):
        B_foreign = False
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(EsperantoStemmer.a_0)
                self.ket = self.cursor
                if among_var == 1:
                    self.slice_from("ĉ")
                elif among_var == 2:
                    self.slice_from("ĝ")
                elif among_var == 3:
                    self.slice_from("ĥ")
                elif among_var == 4:
                    self.slice_from("ĵ")
                elif among_var == 5:
                    self.slice_from("ŝ")
                elif among_var == 6:
                    self.slice_from("ŭ")
                elif among_var == 7:
                    self.slice_from("a")
                    B_foreign = True
                elif among_var == 8:
                    self.slice_from("e")
                    B_foreign = True
                elif among_var == 9:
                    self.slice_from("i")
                    B_foreign = True
                elif among_var == 10:
                    self.slice_from("o")
                    B_foreign = True
                elif among_var == 11:
                    self.slice_from("u")
                    B_foreign = True
                elif among_var == 12:
                    B_foreign = True
                elif among_var == 13:
                    B_foreign = False
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return not B_foreign

    def __r_initial_apostrophe(self):
        self.bra = self.cursor
        if self.cursor == self.limit or self.current[self.cursor] != "'":
            return False
        self.cursor += 1
        self.ket = self.cursor
        if not self.eq_s("st"):
            return False
        if self.find_among(EsperantoStemmer.a_1) == 0:
            return False
        if self.cursor < self.limit:
            return False
        self.slice_from("e")
        return True

    def __r_pronoun(self):
        self.ket = self.cursor
        v_1 = self.limit - self.cursor
        try:
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "n":
                self.cursor = self.limit - v_1
                raise lab0()
            self.cursor -= 1
        except lab0: pass
        self.bra = self.cursor
        if self.find_among_b(EsperantoStemmer.a_2) == 0:
            return False
        while True:
            try:
                if self.cursor > self.limit_backward:
                    raise lab0()
                break
            except lab0: pass
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "-":
                return False
            self.cursor -= 1
            break
        self.slice_del()
        return True

    def __r_final_apostrophe(self):
        self.ket = self.cursor
        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "'":
            return False
        self.cursor -= 1
        self.bra = self.cursor
        while True:
            v_1 = self.limit - self.cursor
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "l":
                    raise lab0()
                self.cursor -= 1
                if self.cursor > self.limit_backward:
                    raise lab0()
                self.slice_from("a")
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                if not self.eq_s_b("un"):
                    raise lab0()
                if self.cursor > self.limit_backward:
                    raise lab0()
                self.slice_from("u")
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                if self.find_among_b(EsperantoStemmer.a_3) == 0:
                    raise lab0()
                while True:
                    try:
                        if self.cursor > self.limit_backward:
                            raise lab1()
                        break
                    except lab1: pass
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "-":
                        raise lab0()
                    self.cursor -= 1
                    break
                self.slice_from("aŭ")
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            self.slice_from("o")
            break
        return True

    def __r_ujn_suffix(self):
        self.ket = self.cursor
        v_1 = self.limit - self.cursor
        try:
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "n":
                self.cursor = self.limit - v_1
                raise lab0()
            self.cursor -= 1
        except lab0: pass
        v_2 = self.limit - self.cursor
        try:
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "j":
                self.cursor = self.limit - v_2
                raise lab0()
            self.cursor -= 1
        except lab0: pass
        self.bra = self.cursor
        if self.find_among_b(EsperantoStemmer.a_4) == 0:
            return False
        while True:
            try:
                if self.cursor > self.limit_backward:
                    raise lab0()
                break
            except lab0: pass
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "-":
                return False
            self.cursor -= 1
            break
        self.slice_del()
        return True

    def __r_uninflected(self):
        if self.find_among_b(EsperantoStemmer.a_5) == 0:
            return False
        while True:
            try:
                if self.cursor > self.limit_backward:
                    raise lab0()
                break
            except lab0: pass
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "-":
                return False
            self.cursor -= 1
            break
        return True

    def __r_merged_numeral(self):
        if self.find_among_b(EsperantoStemmer.a_6) == 0:
            return False
        return self.find_among_b(EsperantoStemmer.a_7) != 0

    def __r_correlative(self):
        self.ket = self.cursor
        self.bra = self.cursor
        v_1 = self.limit - self.cursor
        while True:
            v_2 = self.limit - self.cursor
            try:
                v_3 = self.limit - self.cursor
                try:
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "n":
                        self.cursor = self.limit - v_3
                        raise lab1()
                    self.cursor -= 1
                except lab1: pass
                self.bra = self.cursor
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "e":
                    raise lab0()
                self.cursor -= 1
                break
            except lab0: pass
            self.cursor = self.limit - v_2
            v_4 = self.limit - self.cursor
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "n":
                    self.cursor = self.limit - v_4
                    raise lab0()
                self.cursor -= 1
            except lab0: pass
            v_5 = self.limit - self.cursor
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "j":
                    self.cursor = self.limit - v_5
                    raise lab0()
                self.cursor -= 1
            except lab0: pass
            self.bra = self.cursor
            if not self.in_grouping_b(EsperantoStemmer.g_aou):
                return False
            break
        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "i":
            return False
        self.cursor -= 1
        v_6 = self.limit - self.cursor
        try:
            if self.find_among_b(EsperantoStemmer.a_8) == 0:
                self.cursor = self.limit - v_6
                raise lab0()
        except lab0: pass
        while True:
            try:
                if self.cursor > self.limit_backward:
                    raise lab0()
                break
            except lab0: pass
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "-":
                return False
            self.cursor -= 1
            break
        self.cursor = self.limit - v_1
        self.slice_del()
        return True

    def __r_long_word(self):
        while True:
            v_1 = self.limit - self.cursor
            try:
                for _ in 0, 0:
                    if not self.go_out_grouping_b(EsperantoStemmer.g_vowel):
                        raise lab0()
                    self.cursor -= 1
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            try:
                while True:
                    try:
                        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "-":
                            raise lab1()
                        self.cursor -= 1
                        break
                    except lab1: pass
                    if self.cursor <= self.limit_backward:
                        raise lab0()
                    self.cursor -= 1
                if self.cursor <= self.limit_backward:
                    raise lab0()
                self.cursor -= 1
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            if not self.go_out_grouping_b(EsperantoStemmer.g_digit):
                return False
            self.cursor -= 1
            break
        return True

    def __r_standard_suffix(self):
        self.ket = self.cursor
        among_var = self.find_among_b(EsperantoStemmer.a_9)
        if among_var == 0:
            return False
        if among_var == 1:
            v_1 = self.limit - self.cursor
            while True:
                try:
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "-":
                        raise lab0()
                    self.cursor -= 1
                    break
                except lab0: pass
                if not self.in_grouping_b(EsperantoStemmer.g_digit):
                    return False
                break
            self.cursor = self.limit - v_1
        v_2 = self.limit - self.cursor
        try:
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "-":
                self.cursor = self.limit - v_2
                raise lab0()
            self.cursor -= 1
        except lab0: pass
        self.bra = self.cursor
        self.slice_del()
        return True

    def _stem(self):
        v_1 = self.cursor
        if not self.__r_canonical_form():
            return False
        self.cursor = v_1
        v_2 = self.cursor
        self.__r_initial_apostrophe()
        self.cursor = v_2
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_3 = self.limit - self.cursor
        try:
            if not self.__r_pronoun():
                raise lab0()
            return False
        except lab0: pass
        self.cursor = self.limit - v_3
        v_4 = self.limit - self.cursor
        self.__r_final_apostrophe()
        self.cursor = self.limit - v_4
        v_5 = self.limit - self.cursor
        try:
            if not self.__r_correlative():
                raise lab0()
            return False
        except lab0: pass
        self.cursor = self.limit - v_5
        v_6 = self.limit - self.cursor
        try:
            if not self.__r_uninflected():
                raise lab0()
            return False
        except lab0: pass
        self.cursor = self.limit - v_6
        v_7 = self.limit - self.cursor
        try:
            if not self.__r_merged_numeral():
                raise lab0()
            return False
        except lab0: pass
        self.cursor = self.limit - v_7
        v_8 = self.limit - self.cursor
        try:
            if not self.__r_ujn_suffix():
                raise lab0()
            return False
        except lab0: pass
        self.cursor = self.limit - v_8
        v_9 = self.limit - self.cursor
        if not self.__r_long_word():
            return False
        self.cursor = self.limit - v_9
        if not self.__r_standard_suffix():
            return False
        self.cursor = self.limit_backward
        return True

    a_0 = [
        Among("", -1, 14),
        Among("-", 0, 13),
        Among("cx", 0, 1),
        Among("gx", 0, 2),
        Among("hx", 0, 3),
        Among("jx", 0, 4),
        Among("q", 0, 12),
        Among("sx", 0, 5),
        Among("ux", 0, 6),
        Among("w", 0, 12),
        Among("x", 0, 12),
        Among("y", 0, 12),
        Among("á", 0, 7),
        Among("é", 0, 8),
        Among("í", 0, 9),
        Among("ó", 0, 10),
        Among("ú", 0, 11)
    ]

    a_1 = [
        Among("as", -1, -1),
        Among("i", -1, -1),
        Among("is", 1, -1),
        Among("os", -1, -1),
        Among("u", -1, -1),
        Among("us", 4, -1)
    ]

    a_2 = [
        Among("ci", -1, -1),
        Among("gi", -1, -1),
        Among("hi", -1, -1),
        Among("li", -1, -1),
        Among("ili", 3, -1),
        Among("ŝli", 3, -1),
        Among("mi", -1, -1),
        Among("ni", -1, -1),
        Among("oni", 7, -1),
        Among("ri", -1, -1),
        Among("si", -1, -1),
        Among("vi", -1, -1),
        Among("ivi", 11, -1),
        Among("ĝi", -1, -1),
        Among("ŝi", -1, -1),
        Among("iŝi", 14, -1),
        Among("malŝi", 14, -1)
    ]

    a_3 = [
        Among("amb", -1, -1),
        Among("bald", -1, -1),
        Among("malbald", 1, -1),
        Among("morg", -1, -1),
        Among("postmorg", 3, -1),
        Among("adi", -1, -1),
        Among("hodi", -1, -1),
        Among("ank", -1, -1),
        Among("ĉirk", -1, -1),
        Among("tutĉirk", 8, -1),
        Among("presk", -1, -1),
        Among("almen", -1, -1),
        Among("apen", -1, -1),
        Among("hier", -1, -1),
        Among("antaŭhier", 13, -1),
        Among("malgr", -1, -1),
        Among("ankor", -1, -1),
        Among("kontr", -1, -1),
        Among("anstat", -1, -1),
        Among("kvaz", -1, -1)
    ]

    a_4 = [
        Among("aliu", -1, -1),
        Among("unu", -1, -1)
    ]

    a_5 = [
        Among("aha", -1, -1),
        Among("haha", 0, -1),
        Among("haleluja", -1, -1),
        Among("hola", -1, -1),
        Among("hosana", -1, -1),
        Among("maltra", -1, -1),
        Among("hura", -1, -1),
        Among("ĥaĥa", -1, -1),
        Among("ekde", -1, -1),
        Among("elde", -1, -1),
        Among("disde", -1, -1),
        Among("ehe", -1, -1),
        Among("maltre", -1, -1),
        Among("dirlididi", -1, -1),
        Among("malpli", -1, -1),
        Among("malĉi", -1, -1),
        Among("malkaj", -1, -1),
        Among("amen", -1, -1),
        Among("tamen", 17, -1),
        Among("oho", -1, -1),
        Among("maltro", -1, -1),
        Among("minus", -1, -1),
        Among("uhu", -1, -1),
        Among("muu", -1, -1)
    ]

    a_6 = [
        Among("tri", -1, -1),
        Among("du", -1, -1),
        Among("unu", -1, -1)
    ]

    a_7 = [
        Among("dek", -1, -1),
        Among("cent", -1, -1)
    ]

    a_8 = [
        Among("k", -1, -1),
        Among("kelk", 0, -1),
        Among("nen", -1, -1),
        Among("t", -1, -1),
        Among("mult", 3, -1),
        Among("samt", 3, -1),
        Among("ĉ", -1, -1)
    ]

    a_9 = [
        Among("a", -1, -1),
        Among("e", -1, -1),
        Among("i", -1, -1),
        Among("j", -1, 1),
        Among("aj", 3, -1),
        Among("oj", 3, -1),
        Among("n", -1, 1),
        Among("an", 6, -1),
        Among("en", 6, -1),
        Among("jn", 6, 1),
        Among("ajn", 9, -1),
        Among("ojn", 9, -1),
        Among("on", 6, -1),
        Among("o", -1, -1),
        Among("as", -1, -1),
        Among("is", -1, -1),
        Among("os", -1, -1),
        Among("us", -1, -1),
        Among("u", -1, -1)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass



# Generated from portuguese.sbl by Snowball 3.1.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class PortugueseStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from portuguese.sbl by Snowball 3.1.1 - https://snowballstem.org/
    '''

    g_v = {"a", "e", "i", "o", "u", "á", "â", "é", "ê", "í", "ó", "ô", "ú"}

    I_p2 = 0
    I_p1 = 0
    I_pV = 0

    def __r_prelude(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(PortugueseStemmer.a_0)
                self.ket = self.cursor
                if among_var == 1:
                    self.slice_from("a~")
                elif among_var == 2:
                    self.slice_from("o~")
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_mark_regions(self):
        self.I_pV = self.limit
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            while True:
                v_2 = self.cursor
                try:
                    if not self.in_grouping(PortugueseStemmer.g_v):
                        raise lab1()
                    while True:
                        v_3 = self.cursor
                        try:
                            if not self.out_grouping(PortugueseStemmer.g_v):
                                raise lab2()
                            if not self.go_out_grouping(PortugueseStemmer.g_v):
                                raise lab2()
                            self.cursor += 1
                            break
                        except lab2: pass
                        self.cursor = v_3
                        if not self.in_grouping(PortugueseStemmer.g_v):
                            raise lab1()
                        if not self.go_in_grouping(PortugueseStemmer.g_v):
                            raise lab1()
                        self.cursor += 1
                        break
                    break
                except lab1: pass
                self.cursor = v_2
                if not self.out_grouping(PortugueseStemmer.g_v):
                    raise lab0()
                while True:
                    v_4 = self.cursor
                    try:
                        if not self.out_grouping(PortugueseStemmer.g_v):
                            raise lab1()
                        if not self.go_out_grouping(PortugueseStemmer.g_v):
                            raise lab1()
                        self.cursor += 1
                        break
                    except lab1: pass
                    self.cursor = v_4
                    if not self.in_grouping(PortugueseStemmer.g_v):
                        raise lab0()
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                    break
                break
            self.I_pV = self.cursor
        except lab0: pass
        self.cursor = v_1
        v_5 = self.cursor
        try:
            if not self.go_out_grouping(PortugueseStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(PortugueseStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p1 = self.cursor
            if not self.go_out_grouping(PortugueseStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(PortugueseStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p2 = self.cursor
        except lab0: pass
        self.cursor = v_5
        return True

    def __r_postlude(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(PortugueseStemmer.a_1)
                self.ket = self.cursor
                if among_var == 1:
                    self.slice_from("ã")
                elif among_var == 2:
                    self.slice_from("õ")
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_RV(self):
        return self.I_pV <= self.cursor

    def __r_R2(self):
        return self.I_p2 <= self.cursor

    def __r_standard_suffix(self):
        self.ket = self.cursor
        among_var = self.find_among_b(PortugueseStemmer.a_5)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R2():
                return False
            self.slice_del()
        elif among_var == 2:
            if not self.__r_R2():
                return False
            self.slice_from("log")
        elif among_var == 3:
            if not self.__r_R2():
                return False
            self.slice_from("u")
        elif among_var == 4:
            if not self.__r_R2():
                return False
            self.slice_from("ente")
        elif among_var == 5:
            if self.I_p1 > self.cursor:
                return False
            self.slice_del()
            v_1 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                among_var = self.find_among_b(PortugueseStemmer.a_2)
                if among_var == 0:
                    self.cursor = self.limit - v_1
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_1
                    raise lab0()
                self.slice_del()
                if among_var == 1:
                    self.ket = self.cursor
                    if not self.eq_s_b("at"):
                        self.cursor = self.limit - v_1
                        raise lab0()
                    self.bra = self.cursor
                    if not self.__r_R2():
                        self.cursor = self.limit - v_1
                        raise lab0()
                    self.slice_del()
            except lab0: pass
        elif among_var == 6:
            if not self.__r_R2():
                return False
            self.slice_del()
            v_2 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if self.find_among_b(PortugueseStemmer.a_3) == 0:
                    self.cursor = self.limit - v_2
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_2
                    raise lab0()
                self.slice_del()
            except lab0: pass
        elif among_var == 7:
            if not self.__r_R2():
                return False
            self.slice_del()
            v_3 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if self.find_among_b(PortugueseStemmer.a_4) == 0:
                    self.cursor = self.limit - v_3
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_3
                    raise lab0()
                self.slice_del()
            except lab0: pass
        elif among_var == 8:
            if not self.__r_R2():
                return False
            self.slice_del()
            v_4 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if not self.eq_s_b("at"):
                    self.cursor = self.limit - v_4
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_4
                    raise lab0()
                self.slice_del()
            except lab0: pass
        else:
            if not self.__r_RV():
                return False
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "e":
                return False
            self.cursor -= 1
            self.slice_from("ir")
        return True

    def __r_verb_suffix(self):
        if self.cursor < self.I_pV:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_pV
        self.ket = self.cursor
        if self.find_among_b(PortugueseStemmer.a_6) == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.slice_del()
        self.limit_backward = v_2
        return True

    def __r_residual_suffix(self):
        self.ket = self.cursor
        if self.find_among_b(PortugueseStemmer.a_7) == 0:
            return False
        self.bra = self.cursor
        if not self.__r_RV():
            return False
        self.slice_del()
        return True

    def __r_residual_form(self):
        self.ket = self.cursor
        among_var = self.find_among_b(PortugueseStemmer.a_8)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_RV():
                return False
            self.slice_del()
            self.ket = self.cursor
            while True:
                v_1 = self.limit - self.cursor
                try:
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "u":
                        raise lab0()
                    self.cursor -= 1
                    self.bra = self.cursor
                    v_2 = self.limit - self.cursor
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "g":
                        raise lab0()
                    self.cursor -= 1
                    self.cursor = self.limit - v_2
                    break
                except lab0: pass
                self.cursor = self.limit - v_1
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "i":
                    return False
                self.cursor -= 1
                self.bra = self.cursor
                v_3 = self.limit - self.cursor
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "c":
                    return False
                self.cursor -= 1
                self.cursor = self.limit - v_3
                break
            if not self.__r_RV():
                return False
            self.slice_del()
        else:
            self.slice_from("c")
        return True

    def _stem(self):
        v_1 = self.cursor
        self.__r_prelude()
        self.cursor = v_1
        self.__r_mark_regions()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_2 = self.limit - self.cursor
        try:
            while True:
                v_3 = self.limit - self.cursor
                try:
                    v_4 = self.limit - self.cursor
                    while True:
                        v_5 = self.limit - self.cursor
                        try:
                            if not self.__r_standard_suffix():
                                raise lab2()
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_5
                        if not self.__r_verb_suffix():
                            raise lab1()
                        break
                    self.cursor = self.limit - v_4
                    v_6 = self.limit - self.cursor
                    try:
                        self.ket = self.cursor
                        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "i":
                            raise lab2()
                        self.cursor -= 1
                        self.bra = self.cursor
                        v_7 = self.limit - self.cursor
                        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "c":
                            raise lab2()
                        self.cursor -= 1
                        self.cursor = self.limit - v_7
                        if not self.__r_RV():
                            raise lab2()
                        self.slice_del()
                    except lab2: pass
                    self.cursor = self.limit - v_6
                    break
                except lab1: pass
                self.cursor = self.limit - v_3
                if not self.__r_residual_suffix():
                    raise lab0()
                break
        except lab0: pass
        self.cursor = self.limit - v_2
        v_8 = self.limit - self.cursor
        self.__r_residual_form()
        self.cursor = self.limit - v_8
        self.cursor = self.limit_backward
        v_9 = self.cursor
        self.__r_postlude()
        self.cursor = v_9
        return True

    a_0 = [
        Among("", -1, 3),
        Among("ã", 0, 1),
        Among("õ", 0, 2)
    ]

    a_1 = [
        Among("", -1, 3),
        Among("a~", 0, 1),
        Among("o~", 0, 2)
    ]

    a_2 = [
        Among("ic", -1, -1),
        Among("ad", -1, -1),
        Among("os", -1, -1),
        Among("iv", -1, 1)
    ]

    a_3 = [
        Among("ante", -1, 1),
        Among("avel", -1, 1),
        Among("ível", -1, 1)
    ]

    a_4 = [
        Among("ic", -1, 1),
        Among("abil", -1, 1),
        Among("iv", -1, 1)
    ]

    a_5 = [
        Among("ica", -1, 1),
        Among("ância", -1, 1),
        Among("ência", -1, 4),
        Among("logia", -1, 2),
        Among("ira", -1, 9),
        Among("adora", -1, 1),
        Among("osa", -1, 1),
        Among("ista", -1, 1),
        Among("iva", -1, 8),
        Among("eza", -1, 1),
        Among("idade", -1, 7),
        Among("ante", -1, 1),
        Among("mente", -1, 6),
        Among("amente", 12, 5),
        Among("ável", -1, 1),
        Among("ível", -1, 1),
        Among("ico", -1, 1),
        Among("ismo", -1, 1),
        Among("oso", -1, 1),
        Among("amento", -1, 1),
        Among("imento", -1, 1),
        Among("ivo", -1, 8),
        Among("aça~o", -1, 1),
        Among("uça~o", -1, 3),
        Among("ador", -1, 1),
        Among("icas", -1, 1),
        Among("ências", -1, 4),
        Among("logias", -1, 2),
        Among("iras", -1, 9),
        Among("adoras", -1, 1),
        Among("osas", -1, 1),
        Among("istas", -1, 1),
        Among("ivas", -1, 8),
        Among("ezas", -1, 1),
        Among("idades", -1, 7),
        Among("adores", -1, 1),
        Among("antes", -1, 1),
        Among("aço~es", -1, 1),
        Among("uço~es", -1, 3),
        Among("icos", -1, 1),
        Among("ismos", -1, 1),
        Among("osos", -1, 1),
        Among("amentos", -1, 1),
        Among("imentos", -1, 1),
        Among("ivos", -1, 8)
    ]

    a_6 = [
        Among("ada", -1, 1),
        Among("ida", -1, 1),
        Among("ia", -1, 1),
        Among("aria", 2, 1),
        Among("eria", 2, 1),
        Among("iria", 2, 1),
        Among("ara", -1, 1),
        Among("era", -1, 1),
        Among("ira", -1, 1),
        Among("ava", -1, 1),
        Among("asse", -1, 1),
        Among("esse", -1, 1),
        Among("isse", -1, 1),
        Among("aste", -1, 1),
        Among("este", -1, 1),
        Among("iste", -1, 1),
        Among("ei", -1, 1),
        Among("arei", 16, 1),
        Among("erei", 16, 1),
        Among("irei", 16, 1),
        Among("am", -1, 1),
        Among("iam", 20, 1),
        Among("ariam", 21, 1),
        Among("eriam", 21, 1),
        Among("iriam", 21, 1),
        Among("aram", 20, 1),
        Among("eram", 20, 1),
        Among("iram", 20, 1),
        Among("avam", 20, 1),
        Among("em", -1, 1),
        Among("arem", 29, 1),
        Among("erem", 29, 1),
        Among("irem", 29, 1),
        Among("assem", 29, 1),
        Among("essem", 29, 1),
        Among("issem", 29, 1),
        Among("ado", -1, 1),
        Among("ido", -1, 1),
        Among("ando", -1, 1),
        Among("endo", -1, 1),
        Among("indo", -1, 1),
        Among("ara~o", -1, 1),
        Among("era~o", -1, 1),
        Among("ira~o", -1, 1),
        Among("ar", -1, 1),
        Among("er", -1, 1),
        Among("ir", -1, 1),
        Among("as", -1, 1),
        Among("adas", 47, 1),
        Among("idas", 47, 1),
        Among("ias", 47, 1),
        Among("arias", 50, 1),
        Among("erias", 50, 1),
        Among("irias", 50, 1),
        Among("aras", 47, 1),
        Among("eras", 47, 1),
        Among("iras", 47, 1),
        Among("avas", 47, 1),
        Among("es", -1, 1),
        Among("ardes", 58, 1),
        Among("erdes", 58, 1),
        Among("irdes", 58, 1),
        Among("ares", 58, 1),
        Among("eres", 58, 1),
        Among("ires", 58, 1),
        Among("asses", 58, 1),
        Among("esses", 58, 1),
        Among("isses", 58, 1),
        Among("astes", 58, 1),
        Among("estes", 58, 1),
        Among("istes", 58, 1),
        Among("is", -1, 1),
        Among("ais", 71, 1),
        Among("eis", 71, 1),
        Among("areis", 73, 1),
        Among("ereis", 73, 1),
        Among("ireis", 73, 1),
        Among("áreis", 73, 1),
        Among("éreis", 73, 1),
        Among("íreis", 73, 1),
        Among("ásseis", 73, 1),
        Among("ésseis", 73, 1),
        Among("ísseis", 73, 1),
        Among("áveis", 73, 1),
        Among("íeis", 73, 1),
        Among("aríeis", 84, 1),
        Among("eríeis", 84, 1),
        Among("iríeis", 84, 1),
        Among("ados", -1, 1),
        Among("idos", -1, 1),
        Among("amos", -1, 1),
        Among("áramos", 90, 1),
        Among("éramos", 90, 1),
        Among("íramos", 90, 1),
        Among("ávamos", 90, 1),
        Among("íamos", 90, 1),
        Among("aríamos", 95, 1),
        Among("eríamos", 95, 1),
        Among("iríamos", 95, 1),
        Among("emos", -1, 1),
        Among("aremos", 99, 1),
        Among("eremos", 99, 1),
        Among("iremos", 99, 1),
        Among("ássemos", 99, 1),
        Among("êssemos", 99, 1),
        Among("íssemos", 99, 1),
        Among("imos", -1, 1),
        Among("armos", -1, 1),
        Among("ermos", -1, 1),
        Among("irmos", -1, 1),
        Among("ámos", -1, 1),
        Among("arás", -1, 1),
        Among("erás", -1, 1),
        Among("irás", -1, 1),
        Among("eu", -1, 1),
        Among("iu", -1, 1),
        Among("ou", -1, 1),
        Among("ará", -1, 1),
        Among("erá", -1, 1),
        Among("irá", -1, 1)
    ]

    a_7 = [
        Among("a", -1, 1),
        Among("i", -1, 1),
        Among("o", -1, 1),
        Among("os", -1, 1),
        Among("á", -1, 1),
        Among("í", -1, 1),
        Among("ó", -1, 1)
    ]

    a_8 = [
        Among("e", -1, 1),
        Among("ç", -1, 2),
        Among("é", -1, 1),
        Among("ê", -1, 1)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass



# Generated from dutch.sbl by Snowball 3.1.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class DutchStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from dutch.sbl by Snowball 3.1.1 - https://snowballstem.org/
    '''

    g_E = {"e", "è", "é", "ê", "ë"}

    g_AIOU = {"a", "i", "o", "u", "à", "á", "â", "ä", "ì", "í", "î", "ï", "ò", "ó", "ô", "ö", "ù", "ú", "û", "ü"}

    g_AEIOU = {"a", "e", "i", "o", "u", "à", "á", "â", "ä", "è", "é", "ê", "ë", "ì", "í", "î", "ï", "ò", "ó", "ô", "ö", "ù", "ú", "û", "ü"}

    g_v = {"a", "e", "i", "o", "u", "y", "à", "á", "â", "ä", "è", "é", "ê", "ë", "ì", "í", "î", "ï", "ò", "ó", "ô", "ö", "ù", "ú", "û", "ü"}

    g_v_WX = {"a", "e", "i", "o", "u", "w", "x", "y", "à", "á", "â", "ä", "è", "é", "ê", "ë", "ì", "í", "î", "ï", "ò", "ó", "ô", "ö", "ù", "ú", "û", "ü"}

    B_GE_removed = False
    I_p2 = 0
    I_p1 = 0

    def __r_R1(self):
        return self.I_p1 <= self.cursor

    def __r_R2(self):
        return self.I_p2 <= self.cursor

    def __r_V(self):
        v_1 = self.limit - self.cursor
        while True:
            try:
                if not self.in_grouping_b(DutchStemmer.g_v):
                    raise lab0()
                break
            except lab0: pass
            if not self.eq_s_b("ij"):
                return False
            break
        self.cursor = self.limit - v_1
        return True

    def __r_VX(self):
        v_1 = self.limit - self.cursor
        if self.cursor <= self.limit_backward:
            return False
        self.cursor -= 1
        while True:
            try:
                if not self.in_grouping_b(DutchStemmer.g_v):
                    raise lab0()
                break
            except lab0: pass
            if not self.eq_s_b("ij"):
                return False
            break
        self.cursor = self.limit - v_1
        return True

    def __r_C(self):
        v_1 = self.limit - self.cursor
        try:
            if not self.eq_s_b("ij"):
                raise lab0()
            return False
        except lab0: pass
        if not self.out_grouping_b(DutchStemmer.g_v):
            return False
        self.cursor = self.limit - v_1
        return True

    def __r_lengthen_V(self):
        v_1 = self.limit - self.cursor
        try:
            if not self.out_grouping_b(DutchStemmer.g_v_WX):
                raise lab0()
            self.ket = self.cursor
            among_var = self.find_among_b(DutchStemmer.a_0)
            if among_var == 0:
                raise lab0()
            self.bra = self.cursor
            if among_var == 1:
                v_2 = self.limit - self.cursor
                while True:
                    try:
                        if not self.out_grouping_b(DutchStemmer.g_AEIOU):
                            raise lab1()
                        break
                    except lab1: pass
                    if self.cursor > self.limit_backward:
                        raise lab0()
                    break
                self.cursor = self.limit - v_2
                S_ch = self.slice_to()
                c = self.cursor
                self.insert(self.cursor, self.cursor, S_ch)
                self.cursor = c
            elif among_var == 2:
                v_3 = self.limit - self.cursor
                while True:
                    try:
                        if not self.out_grouping_b(DutchStemmer.g_AEIOU):
                            raise lab1()
                        break
                    except lab1: pass
                    if self.cursor > self.limit_backward:
                        raise lab0()
                    break
                v_4 = self.limit - self.cursor
                try:
                    while True:
                        try:
                            if not self.in_grouping_b(DutchStemmer.g_AIOU):
                                raise lab2()
                            break
                        except lab2: pass
                        if not self.in_grouping_b(DutchStemmer.g_E):
                            raise lab1()
                        if self.cursor > self.limit_backward:
                            raise lab1()
                        break
                    raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_4
                v_5 = self.limit - self.cursor
                try:
                    if self.cursor <= self.limit_backward:
                        raise lab1()
                    self.cursor -= 1
                    if not self.in_grouping_b(DutchStemmer.g_AIOU):
                        raise lab1()
                    if not self.out_grouping_b(DutchStemmer.g_AEIOU):
                        raise lab1()
                    raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_5
                self.cursor = self.limit - v_3
                S_ch = self.slice_to()
                c = self.cursor
                self.insert(self.cursor, self.cursor, S_ch)
                self.cursor = c
            elif among_var == 3:
                self.slice_from("eëe")
            else:
                self.slice_from("iee")
        except lab0: pass
        self.cursor = self.limit - v_1
        return True

    def __r_Step_1(self):
        self.ket = self.cursor
        among_var = self.find_among_b(DutchStemmer.a_1)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            self.slice_del()
        elif among_var == 2:
            if not self.__r_R1():
                return False
            v_1 = self.limit - self.cursor
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "t":
                    raise lab0()
                self.cursor -= 1
                if not self.__r_R1():
                    raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_1
            if not self.__r_C():
                return False
            self.slice_del()
        elif among_var == 3:
            if not self.__r_R1():
                return False
            self.slice_from("ie")
        elif among_var == 4:
            while True:
                v_2 = self.limit - self.cursor
                try:
                    v_3 = self.limit - self.cursor
                    if not self.eq_s_b("ar"):
                        raise lab0()
                    if not self.__r_R1():
                        raise lab0()
                    if not self.__r_C():
                        raise lab0()
                    self.cursor = self.limit - v_3
                    self.slice_del()
                    self.__r_lengthen_V()
                    break
                except lab0: pass
                self.cursor = self.limit - v_2
                try:
                    v_4 = self.limit - self.cursor
                    if not self.eq_s_b("er"):
                        raise lab0()
                    if not self.__r_R1():
                        raise lab0()
                    if not self.__r_C():
                        raise lab0()
                    self.cursor = self.limit - v_4
                    self.slice_del()
                    break
                except lab0: pass
                self.cursor = self.limit - v_2
                if not self.__r_R1():
                    return False
                if not self.__r_C():
                    return False
                self.slice_from("e")
                break
        elif among_var == 5:
            if not self.__r_R1():
                return False
            self.slice_from("é")
        elif among_var == 6:
            if not self.__r_R1():
                return False
            if not self.__r_V():
                return False
            self.slice_from("au")
        elif among_var == 7:
            while True:
                v_5 = self.limit - self.cursor
                try:
                    if not self.eq_s_b("hed"):
                        raise lab0()
                    if not self.__r_R1():
                        raise lab0()
                    self.bra = self.cursor
                    self.slice_from("heid")
                    break
                except lab0: pass
                self.cursor = self.limit - v_5
                try:
                    if not self.eq_s_b("nd"):
                        raise lab0()
                    self.slice_del()
                    break
                except lab0: pass
                self.cursor = self.limit - v_5
                try:
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "d":
                        raise lab0()
                    self.cursor -= 1
                    if not self.__r_R1():
                        raise lab0()
                    if not self.__r_C():
                        raise lab0()
                    self.bra = self.cursor
                    self.slice_del()
                    break
                except lab0: pass
                self.cursor = self.limit - v_5
                try:
                    while True:
                        try:
                            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "i":
                                raise lab1()
                            self.cursor -= 1
                            break
                        except lab1: pass
                        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "j":
                            raise lab0()
                        self.cursor -= 1
                        break
                    if not self.__r_V():
                        raise lab0()
                    self.slice_del()
                    break
                except lab0: pass
                self.cursor = self.limit - v_5
                if not self.__r_R1():
                    return False
                if not self.__r_C():
                    return False
                self.slice_del()
                self.__r_lengthen_V()
                break
        else:
            self.slice_from("nd")
        return True

    def __r_Step_2(self):
        self.ket = self.cursor
        among_var = self.find_among_b(DutchStemmer.a_2)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            while True:
                v_1 = self.limit - self.cursor
                try:
                    if not self.eq_s_b("'t"):
                        raise lab0()
                    self.bra = self.cursor
                    self.slice_del()
                    break
                except lab0: pass
                self.cursor = self.limit - v_1
                try:
                    if not self.eq_s_b("et"):
                        raise lab0()
                    self.bra = self.cursor
                    if not self.__r_R1():
                        raise lab0()
                    if not self.__r_C():
                        raise lab0()
                    self.slice_del()
                    break
                except lab0: pass
                self.cursor = self.limit - v_1
                try:
                    if not self.eq_s_b("rnt"):
                        raise lab0()
                    self.bra = self.cursor
                    self.slice_from("rn")
                    break
                except lab0: pass
                self.cursor = self.limit - v_1
                try:
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "t":
                        raise lab0()
                    self.cursor -= 1
                    self.bra = self.cursor
                    if not self.__r_R1():
                        raise lab0()
                    if not self.__r_VX():
                        raise lab0()
                    self.slice_del()
                    break
                except lab0: pass
                self.cursor = self.limit - v_1
                try:
                    if not self.eq_s_b("ink"):
                        raise lab0()
                    self.bra = self.cursor
                    self.slice_from("ing")
                    break
                except lab0: pass
                self.cursor = self.limit - v_1
                try:
                    if not self.eq_s_b("mp"):
                        raise lab0()
                    self.bra = self.cursor
                    self.slice_from("m")
                    break
                except lab0: pass
                self.cursor = self.limit - v_1
                try:
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "'":
                        raise lab0()
                    self.cursor -= 1
                    self.bra = self.cursor
                    if not self.__r_R1():
                        raise lab0()
                    self.slice_del()
                    break
                except lab0: pass
                self.cursor = self.limit - v_1
                self.bra = self.cursor
                if not self.__r_R1():
                    return False
                if not self.__r_C():
                    return False
                self.slice_del()
                break
        elif among_var == 2:
            if not self.__r_R1():
                return False
            self.slice_from("g")
        elif among_var == 3:
            if not self.__r_R1():
                return False
            self.slice_from("lijk")
        elif among_var == 4:
            if not self.__r_R1():
                return False
            self.slice_from("isch")
        elif among_var == 5:
            if not self.__r_R1():
                return False
            if not self.__r_C():
                return False
            self.slice_del()
        elif among_var == 6:
            if not self.__r_R1():
                return False
            self.slice_from("t")
        elif among_var == 7:
            if not self.__r_R1():
                return False
            self.slice_from("s")
        elif among_var == 8:
            if not self.__r_R1():
                return False
            self.slice_from("r")
        elif among_var == 9:
            if not self.__r_R1():
                return False
            self.slice_del()
            self.insert(self.cursor, self.cursor, "l")
            self.__r_lengthen_V()
        elif among_var == 10:
            if not self.__r_R1():
                return False
            if not self.__r_C():
                return False
            self.slice_del()
            self.insert(self.cursor, self.cursor, "en")
            self.__r_lengthen_V()
        else:
            if not self.__r_R1():
                return False
            if not self.__r_C():
                return False
            self.slice_from("ief")
        return True

    def __r_Step_3(self):
        self.ket = self.cursor
        among_var = self.find_among_b(DutchStemmer.a_3)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R1():
                return False
            self.slice_from("eer")
        elif among_var == 2:
            if not self.__r_R1():
                return False
            self.slice_del()
            self.__r_lengthen_V()
        elif among_var == 3:
            if not self.__r_R1():
                return False
            self.slice_del()
        elif among_var == 4:
            self.slice_from("r")
        elif among_var == 5:
            while True:
                v_1 = self.limit - self.cursor
                try:
                    if not self.eq_s_b("ild"):
                        raise lab0()
                    self.slice_from("er")
                    break
                except lab0: pass
                self.cursor = self.limit - v_1
                if not self.__r_R1():
                    return False
                self.slice_del()
                self.__r_lengthen_V()
                break
        elif among_var == 6:
            if not self.__r_R1():
                return False
            if not self.__r_C():
                return False
            self.slice_from("aar")
        elif among_var == 7:
            if not self.__r_R2():
                return False
            self.slice_del()
            self.insert(self.cursor, self.cursor, "f")
            self.__r_lengthen_V()
        elif among_var == 8:
            if not self.__r_R2():
                return False
            self.slice_del()
            self.insert(self.cursor, self.cursor, "g")
            self.__r_lengthen_V()
        elif among_var == 9:
            if not self.__r_R1():
                return False
            if not self.__r_C():
                return False
            self.slice_from("t")
        else:
            if not self.__r_R1():
                return False
            if not self.__r_C():
                return False
            self.slice_from("d")
        return True

    def __r_Step_4(self):
        while True:
            v_1 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                among_var = self.find_among_b(DutchStemmer.a_4)
                if among_var == 0:
                    raise lab0()
                self.bra = self.cursor
                if among_var == 1:
                    if not self.__r_R1():
                        raise lab0()
                    self.slice_from("ie")
                elif among_var == 2:
                    if not self.__r_R1():
                        raise lab0()
                    self.slice_from("eer")
                elif among_var == 3:
                    if not self.__r_R1():
                        raise lab0()
                    self.slice_del()
                elif among_var == 4:
                    if not self.__r_R1():
                        raise lab0()
                    if not self.__r_V():
                        raise lab0()
                    self.slice_from("n")
                elif among_var == 5:
                    if not self.__r_R1():
                        raise lab0()
                    if not self.__r_V():
                        raise lab0()
                    self.slice_from("l")
                elif among_var == 6:
                    if not self.__r_R1():
                        raise lab0()
                    if not self.__r_V():
                        raise lab0()
                    self.slice_from("r")
                elif among_var == 7:
                    if not self.__r_R1():
                        raise lab0()
                    self.slice_from("teer")
                elif among_var == 8:
                    if not self.__r_R1():
                        raise lab0()
                    self.slice_from("lijk")
                else:
                    if not self.__r_R1():
                        raise lab0()
                    if not self.__r_C():
                        raise lab0()
                    self.slice_del()
                    self.__r_lengthen_V()
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            self.ket = self.cursor
            if self.find_among_b(DutchStemmer.a_5) == 0:
                return False
            self.bra = self.cursor
            if not self.__r_R1():
                return False
            v_2 = self.limit - self.cursor
            try:
                if not self.eq_s_b("inn"):
                    raise lab0()
                if self.cursor > self.limit_backward:
                    raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_2
            if not self.__r_C():
                return False
            self.slice_del()
            self.__r_lengthen_V()
            break
        return True

    def __r_Step_7(self):
        self.ket = self.cursor
        among_var = self.find_among_b(DutchStemmer.a_6)
        if among_var == 0:
            return False
        self.bra = self.cursor
        self.slice_from(DutchStemmer.as_6[among_var - 1])
        return True

    def __r_Step_6(self):
        self.ket = self.cursor
        among_var = self.find_among_b(DutchStemmer.a_7)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            self.slice_from("b")
        elif among_var == 2:
            self.slice_from("c")
        elif among_var == 3:
            self.slice_from("d")
        elif among_var == 4:
            self.slice_from("f")
        elif among_var == 5:
            self.slice_from("g")
        elif among_var == 6:
            self.slice_from("h")
        elif among_var == 7:
            self.slice_from("j")
        elif among_var == 8:
            self.slice_from("k")
        elif among_var == 9:
            self.slice_from("l")
        elif among_var == 10:
            self.slice_from("m")
        elif among_var == 11:
            v_1 = self.limit - self.cursor
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "i":
                    raise lab0()
                self.cursor -= 1
                if self.cursor > self.limit_backward:
                    raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_1
            self.slice_from("n")
        elif among_var == 12:
            self.slice_from("p")
        elif among_var == 13:
            self.slice_from("q")
        elif among_var == 14:
            self.slice_from("r")
        elif among_var == 15:
            self.slice_from("s")
        elif among_var == 16:
            self.slice_from("t")
        elif among_var == 17:
            self.slice_from("v")
        elif among_var == 18:
            self.slice_from("w")
        elif among_var == 19:
            self.slice_from("x")
        else:
            self.slice_from("z")
        return True

    def __r_Step_1c(self):
        self.ket = self.cursor
        among_var = self.find_among_b(DutchStemmer.a_8)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if not self.__r_R1():
            return False
        if not self.__r_C():
            return False
        if among_var == 1:
            v_1 = self.limit - self.cursor
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "n":
                    raise lab0()
                self.cursor -= 1
                if not self.__r_R1():
                    raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_1
            while True:
                v_2 = self.limit - self.cursor
                try:
                    if not self.eq_s_b("in"):
                        raise lab0()
                    if self.cursor > self.limit_backward:
                        raise lab0()
                    self.slice_from("n")
                    break
                except lab0: pass
                self.cursor = self.limit - v_2
                self.slice_del()
                break
        else:
            v_3 = self.limit - self.cursor
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "h":
                    raise lab0()
                self.cursor -= 1
                if not self.__r_R1():
                    raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_3
            v_4 = self.limit - self.cursor
            try:
                if not self.eq_s_b("en"):
                    raise lab0()
                if self.cursor > self.limit_backward:
                    raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_4
            self.slice_del()
        return True

    def __r_Lose_prefix(self):
        self.bra = self.cursor
        if not self.eq_s("ge"):
            return False
        self.ket = self.cursor
        v_1 = self.cursor
        if self.cursor + 3 > self.limit:
            return False
        self.cursor += 3
        self.cursor = v_1
        v_2 = self.cursor
        while True:
            v_3 = self.cursor
            try:
                while True:
                    try:
                        if not self.eq_s("ij"):
                            raise lab1()
                        break
                    except lab1: pass
                    if not self.in_grouping(DutchStemmer.g_v):
                        raise lab0()
                    break
                break
            except lab0: pass
            self.cursor = v_3
            if self.cursor >= self.limit:
                return False
            self.cursor += 1
        while True:
            v_4 = self.cursor
            try:
                while True:
                    try:
                        if not self.eq_s("ij"):
                            raise lab1()
                        break
                    except lab1: pass
                    if not self.in_grouping(DutchStemmer.g_v):
                        raise lab0()
                    break
                continue
            except lab0: pass
            self.cursor = v_4
            break
        if self.cursor >= self.limit:
            return False
        self.cursor = v_2
        among_var = self.find_among(DutchStemmer.a_9)
        if among_var == 1:
            return False
        self.B_GE_removed = True
        self.slice_del()
        v_5 = self.cursor
        try:
            self.bra = self.cursor
            among_var = self.find_among(DutchStemmer.a_10)
            if among_var == 0:
                raise lab0()
            self.ket = self.cursor
            self.slice_from(DutchStemmer.as_10[among_var - 1])
        except lab0: pass
        self.cursor = v_5
        return True

    def __r_Lose_infix(self):
        if self.cursor >= self.limit:
            return False
        self.cursor += 1
        while True:
            try:
                self.bra = self.cursor
                if not self.eq_s("ge"):
                    raise lab0()
                self.ket = self.cursor
                break
            except lab0: pass
            if self.cursor >= self.limit:
                return False
            self.cursor += 1
        v_1 = self.cursor
        if self.cursor + 3 > self.limit:
            return False
        self.cursor += 3
        self.cursor = v_1
        v_2 = self.cursor
        while True:
            v_3 = self.cursor
            try:
                while True:
                    try:
                        if not self.eq_s("ij"):
                            raise lab1()
                        break
                    except lab1: pass
                    if not self.in_grouping(DutchStemmer.g_v):
                        raise lab0()
                    break
                break
            except lab0: pass
            self.cursor = v_3
            if self.cursor >= self.limit:
                return False
            self.cursor += 1
        while True:
            v_4 = self.cursor
            try:
                while True:
                    try:
                        if not self.eq_s("ij"):
                            raise lab1()
                        break
                    except lab1: pass
                    if not self.in_grouping(DutchStemmer.g_v):
                        raise lab0()
                    break
                continue
            except lab0: pass
            self.cursor = v_4
            break
        if self.cursor >= self.limit:
            return False
        self.cursor = v_2
        self.B_GE_removed = True
        self.slice_del()
        v_5 = self.cursor
        try:
            self.bra = self.cursor
            among_var = self.find_among(DutchStemmer.a_11)
            if among_var == 0:
                raise lab0()
            self.ket = self.cursor
            self.slice_from(DutchStemmer.as_11[among_var - 1])
        except lab0: pass
        self.cursor = v_5
        return True

    def __r_measure(self):
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            while True:
                try:
                    if not self.out_grouping(DutchStemmer.g_v):
                        raise lab1()
                    continue
                except lab1: pass
                break
            v_2 = 1
            while True:
                v_3 = self.cursor
                try:
                    while True:
                        try:
                            if not self.eq_s("ij"):
                                raise lab2()
                            break
                        except lab2: pass
                        if not self.in_grouping(DutchStemmer.g_v):
                            raise lab1()
                        break
                    v_2 -= 1
                    continue
                except lab1: pass
                self.cursor = v_3
                break
            if v_2 > 0:
                raise lab0()
            if not self.out_grouping(DutchStemmer.g_v):
                raise lab0()
            self.I_p1 = self.cursor
            while True:
                try:
                    if not self.out_grouping(DutchStemmer.g_v):
                        raise lab1()
                    continue
                except lab1: pass
                break
            v_4 = 1
            while True:
                v_5 = self.cursor
                try:
                    while True:
                        try:
                            if not self.eq_s("ij"):
                                raise lab2()
                            break
                        except lab2: pass
                        if not self.in_grouping(DutchStemmer.g_v):
                            raise lab1()
                        break
                    v_4 -= 1
                    continue
                except lab1: pass
                self.cursor = v_5
                break
            if v_4 > 0:
                raise lab0()
            if not self.out_grouping(DutchStemmer.g_v):
                raise lab0()
            self.I_p2 = self.cursor
        except lab0: pass
        self.cursor = v_1
        return True

    def _stem(self):
        B_stemmed = False
        self.__r_measure()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_1 = self.limit - self.cursor
        try:
            if not self.__r_Step_1():
                raise lab0()
            B_stemmed = True
        except lab0: pass
        self.cursor = self.limit - v_1
        v_2 = self.limit - self.cursor
        try:
            if not self.__r_Step_2():
                raise lab0()
            B_stemmed = True
        except lab0: pass
        self.cursor = self.limit - v_2
        v_3 = self.limit - self.cursor
        try:
            if not self.__r_Step_3():
                raise lab0()
            B_stemmed = True
        except lab0: pass
        self.cursor = self.limit - v_3
        v_4 = self.limit - self.cursor
        try:
            if not self.__r_Step_4():
                raise lab0()
            B_stemmed = True
        except lab0: pass
        self.cursor = self.limit - v_4
        self.cursor = self.limit_backward
        self.B_GE_removed = False
        v_5 = self.cursor
        try:
            v_6 = self.cursor
            if not self.__r_Lose_prefix():
                raise lab0()
            self.cursor = v_6
            self.__r_measure()
        except lab0: pass
        self.cursor = v_5
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_7 = self.limit - self.cursor
        try:
            if not self.B_GE_removed:
                raise lab0()
            B_stemmed = True
            if not self.__r_Step_1c():
                raise lab0()
        except lab0: pass
        self.cursor = self.limit - v_7
        self.cursor = self.limit_backward
        self.B_GE_removed = False
        v_8 = self.cursor
        try:
            v_9 = self.cursor
            if not self.__r_Lose_infix():
                raise lab0()
            self.cursor = v_9
            self.__r_measure()
        except lab0: pass
        self.cursor = v_8
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_10 = self.limit - self.cursor
        try:
            if not self.B_GE_removed:
                raise lab0()
            B_stemmed = True
            if not self.__r_Step_1c():
                raise lab0()
        except lab0: pass
        self.cursor = self.limit - v_10
        self.cursor = self.limit_backward
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_11 = self.limit - self.cursor
        try:
            if not self.__r_Step_7():
                raise lab0()
            B_stemmed = True
        except lab0: pass
        self.cursor = self.limit - v_11
        v_12 = self.limit - self.cursor
        try:
            if not B_stemmed:
                raise lab0()
            if not self.__r_Step_6():
                raise lab0()
        except lab0: pass
        self.cursor = self.limit - v_12
        self.cursor = self.limit_backward
        return True

    a_0 = [
        Among("a", -1, 1),
        Among("e", -1, 2),
        Among("o", -1, 1),
        Among("u", -1, 1),
        Among("à", -1, 1),
        Among("á", -1, 1),
        Among("â", -1, 1),
        Among("ä", -1, 1),
        Among("è", -1, 2),
        Among("é", -1, 2),
        Among("ê", -1, 2),
        Among("eë", -1, 3),
        Among("ië", -1, 4),
        Among("ò", -1, 1),
        Among("ó", -1, 1),
        Among("ô", -1, 1),
        Among("ö", -1, 1),
        Among("ù", -1, 1),
        Among("ú", -1, 1),
        Among("û", -1, 1),
        Among("ü", -1, 1)
    ]

    a_1 = [
        Among("nde", -1, 8),
        Among("en", -1, 7),
        Among("s", -1, 2),
        Among("'s", 2, 1),
        Among("es", 2, 4),
        Among("ies", 4, 3),
        Among("aus", 2, 6),
        Among("és", 2, 5)
    ]

    a_2 = [
        Among("de", -1, 5),
        Among("ge", -1, 2),
        Among("ische", -1, 4),
        Among("je", -1, 1),
        Among("lijke", -1, 3),
        Among("le", -1, 9),
        Among("ene", -1, 10),
        Among("re", -1, 8),
        Among("se", -1, 7),
        Among("te", -1, 6),
        Among("ieve", -1, 11)
    ]

    a_3 = [
        Among("heid", -1, 3),
        Among("fie", -1, 7),
        Among("gie", -1, 8),
        Among("atie", -1, 1),
        Among("isme", -1, 5),
        Among("ing", -1, 5),
        Among("arij", -1, 6),
        Among("erij", -1, 5),
        Among("sel", -1, 3),
        Among("rder", -1, 4),
        Among("ster", -1, 3),
        Among("iteit", -1, 2),
        Among("dst", -1, 10),
        Among("tst", -1, 9)
    ]

    a_4 = [
        Among("end", -1, 9),
        Among("atief", -1, 2),
        Among("erig", -1, 9),
        Among("achtig", -1, 3),
        Among("ioneel", -1, 1),
        Among("baar", -1, 3),
        Among("laar", -1, 5),
        Among("naar", -1, 4),
        Among("raar", -1, 6),
        Among("eriger", -1, 9),
        Among("achtiger", -1, 3),
        Among("lijker", -1, 8),
        Among("tant", -1, 7),
        Among("erigst", -1, 9),
        Among("achtigst", -1, 3),
        Among("lijkst", -1, 8)
    ]

    a_5 = [
        Among("ig", -1, 1),
        Among("iger", -1, 1),
        Among("igst", -1, 1)
    ]

    a_6 = [
        Among("ft", -1, 2),
        Among("kt", -1, 1),
        Among("pt", -1, 3)
    ]
    as_6 = ("k", "f", "p")

    a_7 = [
        Among("bb", -1, 1),
        Among("cc", -1, 2),
        Among("dd", -1, 3),
        Among("ff", -1, 4),
        Among("gg", -1, 5),
        Among("hh", -1, 6),
        Among("jj", -1, 7),
        Among("kk", -1, 8),
        Among("ll", -1, 9),
        Among("mm", -1, 10),
        Among("nn", -1, 11),
        Among("pp", -1, 12),
        Among("qq", -1, 13),
        Among("rr", -1, 14),
        Among("ss", -1, 15),
        Among("tt", -1, 16),
        Among("v", -1, 4),
        Among("vv", 16, 17),
        Among("ww", -1, 18),
        Among("xx", -1, 19),
        Among("z", -1, 15),
        Among("zz", 20, 20)
    ]

    a_8 = [
        Among("d", -1, 1),
        Among("t", -1, 2)
    ]

    a_9 = [
        Among("", -1, -1),
        Among("eft", 0, 1),
        Among("vaa", 0, 1),
        Among("val", 0, 1),
        Among("vali", 3, -1),
        Among("vare", 0, 1)
    ]

    a_10 = [
        Among("ë", -1, 1),
        Among("ï", -1, 2)
    ]
    as_10 = ("e", "i")

    a_11 = [
        Among("ë", -1, 1),
        Among("ï", -1, 2)
    ]
    as_11 = ("e", "i")


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass



# Generated from spanish.sbl by Snowball 3.1.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class SpanishStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from spanish.sbl by Snowball 3.1.1 - https://snowballstem.org/
    '''

    g_v = {"a", "e", "i", "o", "u", "á", "é", "í", "ó", "ú", "ü"}

    I_p2 = 0
    I_p1 = 0
    I_pV = 0

    def __r_mark_regions(self):
        self.I_pV = self.limit
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            while True:
                v_2 = self.cursor
                try:
                    if not self.in_grouping(SpanishStemmer.g_v):
                        raise lab1()
                    while True:
                        v_3 = self.cursor
                        try:
                            if not self.out_grouping(SpanishStemmer.g_v):
                                raise lab2()
                            if not self.go_out_grouping(SpanishStemmer.g_v):
                                raise lab2()
                            self.cursor += 1
                            break
                        except lab2: pass
                        self.cursor = v_3
                        if not self.in_grouping(SpanishStemmer.g_v):
                            raise lab1()
                        if not self.go_in_grouping(SpanishStemmer.g_v):
                            raise lab1()
                        self.cursor += 1
                        break
                    break
                except lab1: pass
                self.cursor = v_2
                if not self.out_grouping(SpanishStemmer.g_v):
                    raise lab0()
                while True:
                    v_4 = self.cursor
                    try:
                        if not self.out_grouping(SpanishStemmer.g_v):
                            raise lab1()
                        if not self.go_out_grouping(SpanishStemmer.g_v):
                            raise lab1()
                        self.cursor += 1
                        break
                    except lab1: pass
                    self.cursor = v_4
                    if not self.in_grouping(SpanishStemmer.g_v):
                        raise lab0()
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                    break
                break
            self.I_pV = self.cursor
        except lab0: pass
        self.cursor = v_1
        v_5 = self.cursor
        try:
            if not self.go_out_grouping(SpanishStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(SpanishStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p1 = self.cursor
            if not self.go_out_grouping(SpanishStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(SpanishStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p2 = self.cursor
        except lab0: pass
        self.cursor = v_5
        return True

    def __r_postlude(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(SpanishStemmer.a_0)
                self.ket = self.cursor
                if among_var == 1:
                    self.slice_from("a")
                elif among_var == 2:
                    self.slice_from("e")
                elif among_var == 3:
                    self.slice_from("i")
                elif among_var == 4:
                    self.slice_from("o")
                elif among_var == 5:
                    self.slice_from("u")
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_RV(self):
        return self.I_pV <= self.cursor

    def __r_R2(self):
        return self.I_p2 <= self.cursor

    def __r_attached_pronoun(self):
        self.ket = self.cursor
        if self.find_among_b(SpanishStemmer.a_1) == 0:
            return False
        self.bra = self.cursor
        among_var = self.find_among_b(SpanishStemmer.a_2)
        if among_var == 0:
            return False
        if not self.__r_RV():
            return False
        if among_var == 1:
            self.bra = self.cursor
            self.slice_from("iendo")
        elif among_var == 2:
            self.bra = self.cursor
            self.slice_from("ando")
        elif among_var == 3:
            self.bra = self.cursor
            self.slice_from("ar")
        elif among_var == 4:
            self.bra = self.cursor
            self.slice_from("er")
        elif among_var == 5:
            self.bra = self.cursor
            self.slice_from("ir")
        elif among_var == 6:
            self.slice_del()
        else:
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "u":
                return False
            self.cursor -= 1
            self.slice_del()
        return True

    def __r_standard_suffix(self):
        self.ket = self.cursor
        among_var = self.find_among_b(SpanishStemmer.a_6)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R2():
                return False
            self.slice_del()
        elif among_var == 2:
            if not self.__r_R2():
                return False
            self.slice_del()
            v_1 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if not self.eq_s_b("ic"):
                    self.cursor = self.limit - v_1
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_1
                    raise lab0()
                self.slice_del()
            except lab0: pass
        elif among_var == 3:
            if not self.__r_R2():
                return False
            self.slice_from("log")
        elif among_var == 4:
            if not self.__r_R2():
                return False
            self.slice_from("u")
        elif among_var == 5:
            if not self.__r_R2():
                return False
            self.slice_from("ente")
        elif among_var == 6:
            if self.I_p1 > self.cursor:
                return False
            self.slice_del()
            v_2 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                among_var = self.find_among_b(SpanishStemmer.a_3)
                if among_var == 0:
                    self.cursor = self.limit - v_2
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_2
                    raise lab0()
                self.slice_del()
                if among_var == 1:
                    self.ket = self.cursor
                    if not self.eq_s_b("at"):
                        self.cursor = self.limit - v_2
                        raise lab0()
                    self.bra = self.cursor
                    if not self.__r_R2():
                        self.cursor = self.limit - v_2
                        raise lab0()
                    self.slice_del()
            except lab0: pass
        elif among_var == 7:
            if not self.__r_R2():
                return False
            self.slice_del()
            v_3 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if self.find_among_b(SpanishStemmer.a_4) == 0:
                    self.cursor = self.limit - v_3
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_3
                    raise lab0()
                self.slice_del()
            except lab0: pass
        elif among_var == 8:
            if not self.__r_R2():
                return False
            self.slice_del()
            v_4 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if self.find_among_b(SpanishStemmer.a_5) == 0:
                    self.cursor = self.limit - v_4
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_4
                    raise lab0()
                self.slice_del()
            except lab0: pass
        else:
            if not self.__r_R2():
                return False
            self.slice_del()
            v_5 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if not self.eq_s_b("at"):
                    self.cursor = self.limit - v_5
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_5
                    raise lab0()
                self.slice_del()
            except lab0: pass
        return True

    def __r_y_verb_suffix(self):
        if self.cursor < self.I_pV:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_pV
        self.ket = self.cursor
        if self.find_among_b(SpanishStemmer.a_7) == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "u":
            return False
        self.cursor -= 1
        self.slice_del()
        return True

    def __r_verb_suffix(self):
        if self.cursor < self.I_pV:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_pV
        self.ket = self.cursor
        among_var = self.find_among_b(SpanishStemmer.a_8)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if among_var == 1:
            v_3 = self.limit - self.cursor
            try:
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "u":
                    self.cursor = self.limit - v_3
                    raise lab0()
                self.cursor -= 1
                v_4 = self.limit - self.cursor
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "g":
                    self.cursor = self.limit - v_3
                    raise lab0()
                self.cursor -= 1
                self.cursor = self.limit - v_4
            except lab0: pass
            self.bra = self.cursor
            self.slice_del()
        else:
            self.slice_del()
        return True

    def __r_residual_suffix(self):
        self.ket = self.cursor
        among_var = self.find_among_b(SpanishStemmer.a_9)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_RV():
                return False
            self.slice_del()
        else:
            if not self.__r_RV():
                return False
            self.slice_del()
            v_1 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "u":
                    self.cursor = self.limit - v_1
                    raise lab0()
                self.cursor -= 1
                self.bra = self.cursor
                v_2 = self.limit - self.cursor
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "g":
                    self.cursor = self.limit - v_1
                    raise lab0()
                self.cursor -= 1
                self.cursor = self.limit - v_2
                if not self.__r_RV():
                    self.cursor = self.limit - v_1
                    raise lab0()
                self.slice_del()
            except lab0: pass
        return True

    def _stem(self):
        self.__r_mark_regions()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_1 = self.limit - self.cursor
        self.__r_attached_pronoun()
        self.cursor = self.limit - v_1
        v_2 = self.limit - self.cursor
        try:
            while True:
                v_3 = self.limit - self.cursor
                try:
                    if not self.__r_standard_suffix():
                        raise lab1()
                    break
                except lab1: pass
                self.cursor = self.limit - v_3
                try:
                    if not self.__r_y_verb_suffix():
                        raise lab1()
                    break
                except lab1: pass
                self.cursor = self.limit - v_3
                if not self.__r_verb_suffix():
                    raise lab0()
                break
        except lab0: pass
        self.cursor = self.limit - v_2
        v_4 = self.limit - self.cursor
        self.__r_residual_suffix()
        self.cursor = self.limit - v_4
        self.cursor = self.limit_backward
        v_5 = self.cursor
        self.__r_postlude()
        self.cursor = v_5
        return True

    a_0 = [
        Among("", -1, 6),
        Among("á", 0, 1),
        Among("é", 0, 2),
        Among("í", 0, 3),
        Among("ó", 0, 4),
        Among("ú", 0, 5)
    ]

    a_1 = [
        Among("la", -1, -1),
        Among("sela", 0, -1),
        Among("le", -1, -1),
        Among("me", -1, -1),
        Among("se", -1, -1),
        Among("lo", -1, -1),
        Among("selo", 5, -1),
        Among("las", -1, -1),
        Among("selas", 7, -1),
        Among("les", -1, -1),
        Among("los", -1, -1),
        Among("selos", 10, -1),
        Among("nos", -1, -1)
    ]

    a_2 = [
        Among("ando", -1, 6),
        Among("iendo", -1, 6),
        Among("yendo", -1, 7),
        Among("ándo", -1, 2),
        Among("iéndo", -1, 1),
        Among("ar", -1, 6),
        Among("er", -1, 6),
        Among("ir", -1, 6),
        Among("ár", -1, 3),
        Among("ér", -1, 4),
        Among("ír", -1, 5)
    ]

    a_3 = [
        Among("ic", -1, -1),
        Among("ad", -1, -1),
        Among("os", -1, -1),
        Among("iv", -1, 1)
    ]

    a_4 = [
        Among("able", -1, 1),
        Among("ible", -1, 1),
        Among("ante", -1, 1)
    ]

    a_5 = [
        Among("ic", -1, 1),
        Among("abil", -1, 1),
        Among("iv", -1, 1)
    ]

    a_6 = [
        Among("ica", -1, 1),
        Among("ancia", -1, 2),
        Among("encia", -1, 5),
        Among("adora", -1, 2),
        Among("osa", -1, 1),
        Among("ista", -1, 1),
        Among("iva", -1, 9),
        Among("anza", -1, 1),
        Among("logía", -1, 3),
        Among("idad", -1, 8),
        Among("able", -1, 1),
        Among("ible", -1, 1),
        Among("ante", -1, 2),
        Among("mente", -1, 7),
        Among("amente", 13, 6),
        Among("acion", -1, 2),
        Among("ucion", -1, 4),
        Among("ación", -1, 2),
        Among("ución", -1, 4),
        Among("ico", -1, 1),
        Among("ismo", -1, 1),
        Among("oso", -1, 1),
        Among("amiento", -1, 1),
        Among("imiento", -1, 1),
        Among("ivo", -1, 9),
        Among("ador", -1, 2),
        Among("icas", -1, 1),
        Among("ancias", -1, 2),
        Among("encias", -1, 5),
        Among("adoras", -1, 2),
        Among("osas", -1, 1),
        Among("istas", -1, 1),
        Among("ivas", -1, 9),
        Among("anzas", -1, 1),
        Among("logías", -1, 3),
        Among("idades", -1, 8),
        Among("ables", -1, 1),
        Among("ibles", -1, 1),
        Among("aciones", -1, 2),
        Among("uciones", -1, 4),
        Among("adores", -1, 2),
        Among("antes", -1, 2),
        Among("icos", -1, 1),
        Among("ismos", -1, 1),
        Among("osos", -1, 1),
        Among("amientos", -1, 1),
        Among("imientos", -1, 1),
        Among("ivos", -1, 9)
    ]

    a_7 = [
        Among("ya", -1, 1),
        Among("ye", -1, 1),
        Among("yan", -1, 1),
        Among("yen", -1, 1),
        Among("yeron", -1, 1),
        Among("yendo", -1, 1),
        Among("yo", -1, 1),
        Among("yas", -1, 1),
        Among("yes", -1, 1),
        Among("yais", -1, 1),
        Among("yamos", -1, 1),
        Among("yó", -1, 1)
    ]

    a_8 = [
        Among("aba", -1, 2),
        Among("ada", -1, 2),
        Among("ida", -1, 2),
        Among("ara", -1, 2),
        Among("iera", -1, 2),
        Among("ía", -1, 2),
        Among("aría", 5, 2),
        Among("ería", 5, 2),
        Among("iría", 5, 2),
        Among("ad", -1, 2),
        Among("ed", -1, 2),
        Among("id", -1, 2),
        Among("ase", -1, 2),
        Among("iese", -1, 2),
        Among("aste", -1, 2),
        Among("iste", -1, 2),
        Among("an", -1, 2),
        Among("aban", 16, 2),
        Among("aran", 16, 2),
        Among("ieran", 16, 2),
        Among("ían", 16, 2),
        Among("arían", 20, 2),
        Among("erían", 20, 2),
        Among("irían", 20, 2),
        Among("en", -1, 1),
        Among("asen", 24, 2),
        Among("iesen", 24, 2),
        Among("aron", -1, 2),
        Among("ieron", -1, 2),
        Among("arán", -1, 2),
        Among("erán", -1, 2),
        Among("irán", -1, 2),
        Among("ado", -1, 2),
        Among("ido", -1, 2),
        Among("ando", -1, 2),
        Among("iendo", -1, 2),
        Among("ar", -1, 2),
        Among("er", -1, 2),
        Among("ir", -1, 2),
        Among("as", -1, 2),
        Among("abas", 39, 2),
        Among("adas", 39, 2),
        Among("idas", 39, 2),
        Among("aras", 39, 2),
        Among("ieras", 39, 2),
        Among("ías", 39, 2),
        Among("arías", 45, 2),
        Among("erías", 45, 2),
        Among("irías", 45, 2),
        Among("es", -1, 1),
        Among("ases", 49, 2),
        Among("ieses", 49, 2),
        Among("abais", -1, 2),
        Among("arais", -1, 2),
        Among("ierais", -1, 2),
        Among("íais", -1, 2),
        Among("aríais", 55, 2),
        Among("eríais", 55, 2),
        Among("iríais", 55, 2),
        Among("aseis", -1, 2),
        Among("ieseis", -1, 2),
        Among("asteis", -1, 2),
        Among("isteis", -1, 2),
        Among("áis", -1, 2),
        Among("éis", -1, 1),
        Among("aréis", 64, 2),
        Among("eréis", 64, 2),
        Among("iréis", 64, 2),
        Among("ados", -1, 2),
        Among("idos", -1, 2),
        Among("amos", -1, 2),
        Among("ábamos", 70, 2),
        Among("áramos", 70, 2),
        Among("iéramos", 70, 2),
        Among("íamos", 70, 2),
        Among("aríamos", 74, 2),
        Among("eríamos", 74, 2),
        Among("iríamos", 74, 2),
        Among("emos", -1, 1),
        Among("aremos", 78, 2),
        Among("eremos", 78, 2),
        Among("iremos", 78, 2),
        Among("ásemos", 78, 2),
        Among("iésemos", 78, 2),
        Among("imos", -1, 2),
        Among("arás", -1, 2),
        Among("erás", -1, 2),
        Among("irás", -1, 2),
        Among("ís", -1, 2),
        Among("ará", -1, 2),
        Among("erá", -1, 2),
        Among("irá", -1, 2),
        Among("aré", -1, 2),
        Among("eré", -1, 2),
        Among("iré", -1, 2),
        Among("ió", -1, 2)
    ]

    a_9 = [
        Among("a", -1, 1),
        Among("e", -1, 2),
        Among("o", -1, 1),
        Among("os", -1, 1),
        Among("á", -1, 1),
        Among("é", -1, 2),
        Among("í", -1, 1),
        Among("ó", -1, 1)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass



# Generated from lithuanian.sbl by Snowball 3.1.1 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class LithuanianStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from lithuanian.sbl by Snowball 3.1.1 - https://snowballstem.org/
    '''

    g_v = {"a", "e", "i", "o", "u", "y", "ą", "ė", "ę", "į", "ū", "ų"}

    I_p1 = 0

    def __r_step1(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        if self.find_among_b(LithuanianStemmer.a_0) == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        self.slice_del()
        return True

    def __r_step2(self):
        while True:
            v_1 = self.limit - self.cursor
            try:
                if self.cursor < self.I_p1:
                    raise lab0()
                v_3 = self.limit_backward
                self.limit_backward = self.I_p1
                self.ket = self.cursor
                if self.find_among_b(LithuanianStemmer.a_1) == 0:
                    self.limit_backward = v_3
                    raise lab0()
                self.bra = self.cursor
                self.limit_backward = v_3
                self.slice_del()
                continue
            except lab0: pass
            self.cursor = self.limit - v_1
            break
        return True

    def __r_fix_conflicts(self):
        self.ket = self.cursor
        among_var = self.find_among_b(LithuanianStemmer.a_2)
        if among_var == 0:
            return False
        self.bra = self.cursor
        self.slice_from(LithuanianStemmer.as_2[among_var - 1])
        return True

    def __r_fix_chdz(self):
        self.ket = self.cursor
        among_var = self.find_among_b(LithuanianStemmer.a_3)
        if among_var == 0:
            return False
        self.bra = self.cursor
        self.slice_from(LithuanianStemmer.as_3[among_var - 1])
        return True

    def __r_fix_gd(self):
        self.ket = self.cursor
        if not self.eq_s_b("gd"):
            return False
        self.bra = self.cursor
        self.slice_from("g")
        return True

    def _stem(self):
        self.I_p1 = self.limit
        v_1 = self.cursor
        try:
            v_2 = self.cursor
            try:
                if self.cursor == self.limit or self.current[self.cursor] != "a":
                    self.cursor = v_2
                    raise lab1()
                self.cursor += 1
                if len(self.current) <= 6:
                    self.cursor = v_2
                    raise lab1()
            except lab1: pass
            if not self.go_out_grouping(LithuanianStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(LithuanianStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p1 = self.cursor
        except lab0: pass
        self.cursor = v_1
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_3 = self.limit - self.cursor
        self.__r_fix_conflicts()
        self.cursor = self.limit - v_3
        v_4 = self.limit - self.cursor
        self.__r_step1()
        self.cursor = self.limit - v_4
        v_5 = self.limit - self.cursor
        self.__r_fix_chdz()
        self.cursor = self.limit - v_5
        v_6 = self.limit - self.cursor
        self.__r_step2()
        self.cursor = self.limit - v_6
        v_7 = self.limit - self.cursor
        self.__r_fix_chdz()
        self.cursor = self.limit - v_7
        v_8 = self.limit - self.cursor
        self.__r_fix_gd()
        self.cursor = self.limit - v_8
        self.ket = self.cursor
        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "'":
            return False
        self.cursor -= 1
        self.bra = self.cursor
        self.slice_del()
        self.cursor = self.limit_backward
        return True

    a_0 = [
        Among("a", -1, -1),
        Among("ia", 0, -1),
        Among("osna", 0, -1),
        Among("iosna", 2, -1),
        Among("uosna", 2, -1),
        Among("iuosna", 4, -1),
        Among("ysna", 0, -1),
        Among("ėsna", 0, -1),
        Among("e", -1, -1),
        Among("ie", 8, -1),
        Among("enie", 9, -1),
        Among("oje", 8, -1),
        Among("ioje", 11, -1),
        Among("uje", 8, -1),
        Among("iuje", 13, -1),
        Among("yje", 8, -1),
        Among("enyje", 15, -1),
        Among("ėje", 8, -1),
        Among("ame", 8, -1),
        Among("iame", 18, -1),
        Among("sime", 8, -1),
        Among("ome", 8, -1),
        Among("ėme", 8, -1),
        Among("tumėme", 22, -1),
        Among("ose", 8, -1),
        Among("iose", 24, -1),
        Among("uose", 24, -1),
        Among("iuose", 26, -1),
        Among("yse", 8, -1),
        Among("enyse", 28, -1),
        Among("ėse", 8, -1),
        Among("ate", 8, -1),
        Among("iate", 31, -1),
        Among("ite", 8, -1),
        Among("kite", 33, -1),
        Among("site", 33, -1),
        Among("ote", 8, -1),
        Among("tute", 8, -1),
        Among("ėte", 8, -1),
        Among("tumėte", 38, -1),
        Among("i", -1, -1),
        Among("ai", 40, -1),
        Among("iai", 41, -1),
        Among("ei", 40, -1),
        Among("tumei", 43, -1),
        Among("ki", 40, -1),
        Among("imi", 40, -1),
        Among("umi", 40, -1),
        Among("iumi", 47, -1),
        Among("si", 40, -1),
        Among("asi", 49, -1),
        Among("iasi", 50, -1),
        Among("esi", 49, -1),
        Among("iesi", 52, -1),
        Among("siesi", 53, -1),
        Among("isi", 49, -1),
        Among("aisi", 55, -1),
        Among("eisi", 55, -1),
        Among("tumeisi", 57, -1),
        Among("uisi", 55, -1),
        Among("osi", 49, -1),
        Among("ėjosi", 60, -1),
        Among("uosi", 60, -1),
        Among("iuosi", 62, -1),
        Among("siuosi", 63, -1),
        Among("usi", 49, -1),
        Among("ausi", 65, -1),
        Among("čiausi", 66, -1),
        Among("ąsi", 49, -1),
        Among("ėsi", 49, -1),
        Among("ųsi", 49, -1),
        Among("tųsi", 70, -1),
        Among("ti", 40, -1),
        Among("enti", 72, -1),
        Among("inti", 72, -1),
        Among("oti", 72, -1),
        Among("ioti", 75, -1),
        Among("uoti", 75, -1),
        Among("iuoti", 77, -1),
        Among("auti", 72, -1),
        Among("iauti", 79, -1),
        Among("yti", 72, -1),
        Among("ėti", 72, -1),
        Among("telėti", 82, -1),
        Among("inėti", 82, -1),
        Among("terėti", 82, -1),
        Among("ui", 40, -1),
        Among("iui", 86, -1),
        Among("eniui", 87, -1),
        Among("oj", -1, -1),
        Among("ėj", -1, -1),
        Among("k", -1, -1),
        Among("am", -1, -1),
        Among("iam", 92, -1),
        Among("iem", -1, -1),
        Among("im", -1, -1),
        Among("sim", 95, -1),
        Among("om", -1, -1),
        Among("tum", -1, -1),
        Among("ėm", -1, -1),
        Among("tumėm", 99, -1),
        Among("an", -1, -1),
        Among("on", -1, -1),
        Among("ion", 102, -1),
        Among("un", -1, -1),
        Among("iun", 104, -1),
        Among("ėn", -1, -1),
        Among("o", -1, -1),
        Among("io", 107, -1),
        Among("enio", 108, -1),
        Among("ėjo", 107, -1),
        Among("uo", 107, -1),
        Among("s", -1, -1),
        Among("as", 112, -1),
        Among("ias", 113, -1),
        Among("es", 112, -1),
        Among("ies", 115, -1),
        Among("is", 112, -1),
        Among("ais", 117, -1),
        Among("iais", 118, -1),
        Among("tumeis", 117, -1),
        Among("imis", 117, -1),
        Among("enimis", 121, -1),
        Among("omis", 117, -1),
        Among("iomis", 123, -1),
        Among("umis", 117, -1),
        Among("ėmis", 117, -1),
        Among("enis", 117, -1),
        Among("asis", 117, -1),
        Among("ysis", 117, -1),
        Among("ams", 112, -1),
        Among("iams", 130, -1),
        Among("iems", 112, -1),
        Among("ims", 112, -1),
        Among("enims", 133, -1),
        Among("oms", 112, -1),
        Among("ioms", 135, -1),
        Among("ums", 112, -1),
        Among("ėms", 112, -1),
        Among("ens", 112, -1),
        Among("os", 112, -1),
        Among("ios", 140, -1),
        Among("uos", 140, -1),
        Among("iuos", 142, -1),
        Among("us", 112, -1),
        Among("aus", 144, -1),
        Among("iaus", 145, -1),
        Among("ius", 144, -1),
        Among("ys", 112, -1),
        Among("enys", 148, -1),
        Among("ąs", 112, -1),
        Among("iąs", 150, -1),
        Among("ės", 112, -1),
        Among("amės", 152, -1),
        Among("iamės", 153, -1),
        Among("imės", 152, -1),
        Among("kimės", 155, -1),
        Among("simės", 155, -1),
        Among("omės", 152, -1),
        Among("ėmės", 152, -1),
        Among("tumėmės", 159, -1),
        Among("atės", 152, -1),
        Among("iatės", 161, -1),
        Among("sitės", 152, -1),
        Among("otės", 152, -1),
        Among("ėtės", 152, -1),
        Among("tumėtės", 165, -1),
        Among("įs", 112, -1),
        Among("ūs", 112, -1),
        Among("tųs", 112, -1),
        Among("at", -1, -1),
        Among("iat", 170, -1),
        Among("it", -1, -1),
        Among("sit", 172, -1),
        Among("ot", -1, -1),
        Among("ėt", -1, -1),
        Among("tumėt", 175, -1),
        Among("u", -1, -1),
        Among("au", 177, -1),
        Among("iau", 178, -1),
        Among("čiau", 179, -1),
        Among("iu", 177, -1),
        Among("eniu", 181, -1),
        Among("siu", 181, -1),
        Among("y", -1, -1),
        Among("ą", -1, -1),
        Among("ią", 185, -1),
        Among("ė", -1, -1),
        Among("ę", -1, -1),
        Among("į", -1, -1),
        Among("enį", 189, -1),
        Among("ų", -1, -1),
        Among("ių", 191, -1)
    ]

    a_1 = [
        Among("ing", -1, -1),
        Among("aj", -1, -1),
        Among("iaj", 1, -1),
        Among("iej", -1, -1),
        Among("oj", -1, -1),
        Among("ioj", 4, -1),
        Among("uoj", 4, -1),
        Among("iuoj", 6, -1),
        Among("auj", -1, -1),
        Among("ąj", -1, -1),
        Among("iąj", 9, -1),
        Among("ėj", -1, -1),
        Among("ųj", -1, -1),
        Among("iųj", 12, -1),
        Among("ok", -1, -1),
        Among("iok", 14, -1),
        Among("iuk", -1, -1),
        Among("uliuk", 16, -1),
        Among("učiuk", 16, -1),
        Among("išk", -1, -1),
        Among("iul", -1, -1),
        Among("yl", -1, -1),
        Among("ėl", -1, -1),
        Among("am", -1, -1),
        Among("dam", 23, -1),
        Among("jam", 23, -1),
        Among("zgan", -1, -1),
        Among("ain", -1, -1),
        Among("esn", -1, -1),
        Among("op", -1, -1),
        Among("iop", 29, -1),
        Among("ias", -1, -1),
        Among("ies", -1, -1),
        Among("ais", -1, -1),
        Among("iais", 33, -1),
        Among("os", -1, -1),
        Among("ios", 35, -1),
        Among("uos", 35, -1),
        Among("iuos", 37, -1),
        Among("aus", -1, -1),
        Among("iaus", 39, -1),
        Among("ąs", -1, -1),
        Among("iąs", 41, -1),
        Among("ęs", -1, -1),
        Among("utėait", -1, -1),
        Among("ant", -1, -1),
        Among("iant", 45, -1),
        Among("siant", 46, -1),
        Among("int", -1, -1),
        Among("ot", -1, -1),
        Among("uot", 49, -1),
        Among("iuot", 50, -1),
        Among("yt", -1, -1),
        Among("ėt", -1, -1),
        Among("ykšt", -1, -1),
        Among("iau", -1, -1),
        Among("dav", -1, -1),
        Among("sv", -1, -1),
        Among("šv", -1, -1),
        Among("ykšč", -1, -1),
        Among("ę", -1, -1),
        Among("ėję", 60, -1)
    ]

    a_2 = [
        Among("ojime", -1, 7),
        Among("ėjime", -1, 3),
        Among("avime", -1, 6),
        Among("okate", -1, 8),
        Among("aite", -1, 1),
        Among("uote", -1, 2),
        Among("asius", -1, 5),
        Among("okatės", -1, 8),
        Among("aitės", -1, 1),
        Among("uotės", -1, 2),
        Among("esiu", -1, 4)
    ]
    as_2 = ("aitė", "uotė", "ėjimas", "esys", "asys", "avimas", "ojimas", "okatė")

    a_3 = [
        Among("č", -1, 1),
        Among("dž", -1, 2)
    ]
    as_3 = ("t", "d")


class lab0(BaseException): pass


class lab1(BaseException): pass
